@@ -20,19 +20,23 @@ import (
 
 // These variables are populated via the Go linker.
 var (
-	version string = "0.9"
-	commit  string
-	branch  string
+	version   = "0.9"
+	commit    string
+	branch    string
+	buildTime string
 )
 
 func init() {
-	// If commit or branch are not set, make that clear.
+	// If commit, branch, or build time are not set, make that clear.
 	if commit == "" {
 		commit = "unknown"
 	}
 	if branch == "" {
 		branch = "unknown"
 	}
+	if buildTime == "" {
+		buildTime = "unknown"
+	}
 }
 
 func main() {
@@ -77,6 +81,7 @@ func (m *Main) Run(args ...string) error {
 		cmd.Version = version
 		cmd.Commit = commit
 		cmd.Branch = branch
+		cmd.BuildTime = buildTime
 
 		if err := cmd.Run(args...); err != nil {
 			return fmt.Errorf("run: %s", err)
@@ -164,7 +169,7 @@ func ParseCommandName(args []string) (string, []string) {
 	return "", args
 }
 
-// Command represents the command executed by "influxd version".
+// VersionCommand represents the command executed by "influxd version".
 type VersionCommand struct {
 	Stdout io.Writer
 	Stderr io.Writer
@@ -188,7 +193,7 @@ func (cmd *VersionCommand) Run(args ...string) error {
 	}
 
 	// Print version info.
-	fmt.Fprintf(cmd.Stdout, "InfluxDB v%s (git: %s %s)\n", version, branch, commit)
+	fmt.Fprintf(cmd.Stdout, "InfluxDB v%s (git: %s %s, built %s)\n", version, branch, commit, buildTime)
 
 	return nil
 }
