@@ -7,6 +7,7 @@ import (
 	"io"
 	"math"
 	"regexp"
+	"sort"
 	"strconv"
 	"strings"
 	"time"
@@ -118,7 +119,7 @@ func (p *Parser) parseShowStatement() (Statement, error) {
 		if tok == KEYS {
 			return p.parseShowFieldKeysStatement()
 		}
-		return nil, newParseError(tokstr(tok, lit), []string{"KEYS", "VALUES"}, pos)
+		return nil, newParseError(tokstr(tok, lit), []string{"KEYS"}, pos)
 	case MEASUREMENTS:
 		return p.parseShowMeasurementsStatement()
 	case RETENTION:
@@ -129,6 +130,12 @@ func (p *Parser) parseShowStatement() (Statement, error) {
 		return nil, newParseError(tokstr(tok, lit), []string{"POLICIES"}, pos)
 	case SERIES:
 		return p.parseShowSeriesStatement()
+	case SHARD:
+		tok, pos, lit := p.scanIgnoreWhitespace()
+		if tok == GROUPS {
+			return p.parseShowShardGroupsStatement()
+		}
+		return nil, newParseError(tokstr(tok, lit), []string{"GROUPS"}, pos)
 	case SHARDS:
 		return p.parseShowShardsStatement()
 	case STATS:
@@ -145,9 +152,30 @@ func (p *Parser) parseShowStatement() (Statement, error) {
 		return nil, newParseError(tokstr(tok, lit), []string{"KEYS", "VALUES"}, pos)
 	case USERS:
 		return p.parseShowUsersStatement()
+	case SUBSCRIPTIONS:
+		return p.parseShowSubscriptionsStatement()
 	}
 
-	return nil, newParseError(tokstr(tok, lit), []string{"CONTINUOUS", "DATABASES", "FIELD", "GRANTS", "MEASUREMENTS", "RETENTION", "SERIES", "SERVERS", "TAG", "USERS"}, pos)
+	showQueryKeywords := []string{
+		"CONTINUOUS",
+		"DATABASES",
+		"FIELD",
+		"GRANTS",
+		"MEASUREMENTS",
+		"RETENTION",
+		"SERIES",
+		"SERVERS",
+		"TAG",
+		"USERS",
+		"STATS",
+		"DIAGNOSTICS",
+		"SHARD",
+		"SHARDS",
+		"SUBSCRIPTIONS",
+	}
+	sort.Strings(showQueryKeywords)
+
+	return nil, newParseError(tokstr(tok, lit), showQueryKeywords, pos)
 }
 
 // parseCreateStatement parses a string and returns a create statement.
@@ -166,9 +194,11 @@ func (p *Parser) parseCreateStatement() (Statement, error) {
 			return nil, newParseError(tokstr(tok, lit), []string{"POLICY"}, pos)
 		}
 		return p.parseCreateRetentionPolicyStatement()
+	} else if tok == SUBSCRIPTION {
+		return p.parseCreateSubscriptionStatement()
 	}
 
-	return nil, newParseError(tokstr(tok, lit), []string{"CONTINUOUS", "DATABASE", "USER", "RETENTION"}, pos)
+	return nil, newParseError(tokstr(tok, lit), []string{"CONTINUOUS", "DATABASE", "USER", "RETENTION", "SUBSCRIPTION"}, pos)
 }
 
 // parseDropStatement parses a string and returns a drop statement.
@@ -190,9 +220,13 @@ func (p *Parser) parseDropStatement() (Statement, error) {
 		return p.parseDropRetentionPolicyStatement()
 	} else if tok == USER {
 		return p.parseDropUserStatement()
+	} else if tok == META || tok == DATA {
+		return p.parseDropServerStatement(tok)
+	} else if tok == SUBSCRIPTION {
+		return p.parseDropSubscriptionStatement()
 	}
 
-	return nil, newParseError(tokstr(tok, lit), []string{"SERIES", "CONTINUOUS", "MEASUREMENT"}, pos)
+	return nil, newParseError(tokstr(tok, lit), []string{"SERIES", "CONTINUOUS", "MEASUREMENT", "SERVER", "SUBSCRIPTION"}, pos)
 }
 
 // parseAlterStatement parses a string and returns an alter statement.
@@ -241,6 +275,61 @@ func (p *Parser) parseSetPasswordUserStatement() (*SetPasswordUserStatement, err
 	return stmt, nil
 }
 
+// parseCreateSubscriptionStatement parses a string and returns a CreatesubScriptionStatement.
+// This function assumes the "CREATE SUBSCRIPTION" tokens have already been consumed.
+func (p *Parser) parseCreateSubscriptionStatement() (*CreateSubscriptionStatement, error) {
+	stmt := &CreateSubscriptionStatement{}
+
+	// Read the id of the subscription to create.
+	ident, err := p.parseIdent()
+	if err != nil {
+		return nil, err
+	}
+	stmt.Name = ident
+
+	// Expect an "ON" keyword.
+	if tok, pos, lit := p.scanIgnoreWhitespace(); tok != ON {
+		return nil, newParseError(tokstr(tok, lit), []string{"ON"}, pos)
+	}
+
+	// Read the name of the database.
+	if ident, err = p.parseIdent(); err != nil {
+		return nil, err
+	}
+	stmt.Database = ident
+
+	if tok, pos, lit := p.scan(); tok != DOT {
+		return nil, newParseError(tokstr(tok, lit), []string{"."}, pos)
+	}
+
+	// Read the name of the retention policy.
+	if ident, err = p.parseIdent(); err != nil {
+		return nil, err
+	}
+	stmt.RetentionPolicy = ident
+
+	// Expect a "DESTINATIONS" keyword.
+	if tok, pos, lit := p.scanIgnoreWhitespace(); tok != DESTINATIONS {
+		return nil, newParseError(tokstr(tok, lit), []string{"DESTINATIONS"}, pos)
+	}
+
+	// Expect one of "ANY ALL" keywords.
+	if tok, pos, lit := p.scanIgnoreWhitespace(); tok == ALL || tok == ANY {
+		stmt.Mode = tokens[tok]
+	} else {
+		return nil, newParseError(tokstr(tok, lit), []string{"ALL", "ANY"}, pos)
+	}
+
+	// Read list of destinations.
+	var destinations []string
+	if destinations, err = p.parseStringList(); err != nil {
+		return nil, err
+	}
+	stmt.Destinations = destinations
+
+	return stmt, nil
+}
+
 // parseCreateRetentionPolicyStatement parses a string and returns a create retention policy statement.
 // This function assumes the CREATE RETENTION POLICY tokens have already been consumed.
 func (p *Parser) parseCreateRetentionPolicyStatement() (*CreateRetentionPolicyStatement, error) {
@@ -293,8 +382,8 @@ func (p *Parser) parseCreateRetentionPolicyStatement() (*CreateRetentionPolicySt
 	// Parse optional DEFAULT token.
 	if tok, pos, lit = p.scanIgnoreWhitespace(); tok == DEFAULT {
 		stmt.Default = true
-	} else {
-		p.unscan()
+	} else if tok != EOF && tok != SEMICOLON {
+		return nil, newParseError(tokstr(tok, lit), []string{"DEFAULT"}, pos)
 	}
 
 	return stmt, nil
@@ -524,6 +613,30 @@ func (p *Parser) parseString() (string, error) {
 	return lit, nil
 }
 
+// parserString parses a string.
+func (p *Parser) parseStringList() ([]string, error) {
+	// Parse first (required) string.
+	str, err := p.parseString()
+	if err != nil {
+		return nil, err
+	}
+	strs := []string{str}
+
+	// Parse remaining (optional) strings.
+	for {
+		if tok, _, _ := p.scanIgnoreWhitespace(); tok != COMMA {
+			p.unscan()
+			return strs, nil
+		}
+
+		if str, err = p.parseString(); err != nil {
+			return nil, err
+		}
+
+		strs = append(strs, str)
+	}
+}
+
 // parseRevokeStatement parses a string and returns a revoke statement.
 // This function assumes the REVOKE token has already been consumed.
 func (p *Parser) parseRevokeStatement() (Statement, error) {
@@ -835,26 +948,30 @@ func (p *Parser) parseTarget(tr targetRequirement) (*Target, error) {
 // parseDeleteStatement parses a delete string and returns a DeleteStatement.
 // This function assumes the DELETE token has already been consumed.
 func (p *Parser) parseDeleteStatement() (*DeleteStatement, error) {
-	stmt := &DeleteStatement{}
+	// TODO remove and do not skip test once we wire up DELETE FROM.
+	// See issues https://github.com/influxdb/influxdb/issues/1647
+	// and https://github.com/influxdb/influxdb/issues/4404
+	return nil, errors.New("DELETE FROM is currently not supported. Use DROP SERIES or DROP MEASUREMENT instead")
+	//stmt := &DeleteStatement{}
 
-	// Parse source
-	if tok, pos, lit := p.scanIgnoreWhitespace(); tok != FROM {
-		return nil, newParseError(tokstr(tok, lit), []string{"FROM"}, pos)
-	}
-	source, err := p.parseSource()
-	if err != nil {
-		return nil, err
-	}
-	stmt.Source = source
+	//// Parse source
+	//if tok, pos, lit := p.scanIgnoreWhitespace(); tok != FROM {
+	//	return nil, newParseError(tokstr(tok, lit), []string{"FROM"}, pos)
+	//}
+	//source, err := p.parseSource()
+	//if err != nil {
+	//	return nil, err
+	//}
+	//stmt.Source = source
 
-	// Parse condition: "WHERE EXPR".
-	condition, err := p.parseCondition()
-	if err != nil {
-		return nil, err
-	}
-	stmt.Condition = condition
+	//// Parse condition: "WHERE EXPR".
+	//condition, err := p.parseCondition()
+	//if err != nil {
+	//	return nil, err
+	//}
+	//stmt.Condition = condition
 
-	return stmt, nil
+	//return stmt, nil
 }
 
 // parseShowSeriesStatement parses a string and returns a ShowSeriesStatement.
@@ -901,6 +1018,29 @@ func (p *Parser) parseShowMeasurementsStatement() (*ShowMeasurementsStatement, e
 	stmt := &ShowMeasurementsStatement{}
 	var err error
 
+	// Parse optional WITH clause.
+	if tok, _, _ := p.scanIgnoreWhitespace(); tok == WITH {
+		// Parse required MEASUREMENT token.
+		if err := p.parseTokens([]Token{MEASUREMENT}); err != nil {
+			return nil, err
+		}
+
+		// Parse required operator: = or =~.
+		tok, pos, lit := p.scanIgnoreWhitespace()
+		switch tok {
+		case EQ, EQREGEX:
+			// Parse required source (measurement name or regex).
+			if stmt.Source, err = p.parseSource(); err != nil {
+				return nil, err
+			}
+		default:
+			return nil, newParseError(tokstr(tok, lit), []string{"=", "=~"}, pos)
+		}
+	} else {
+		// Not a WITH clause so put the token back.
+		p.unscan()
+	}
+
 	// Parse condition: "WHERE EXPR".
 	if stmt.Condition, err = p.parseCondition(); err != nil {
 		return nil, err
@@ -979,6 +1119,16 @@ func (p *Parser) parseShowTagKeysStatement() (*ShowTagKeysStatement, error) {
 		return nil, err
 	}
 
+	// Parse series limit: "SLIMIT <n>".
+	if stmt.SLimit, err = p.parseOptionalTokenAndInt(SLIMIT); err != nil {
+		return nil, err
+	}
+
+	// Parse series offset: "SOFFSET <n>".
+	if stmt.SOffset, err = p.parseOptionalTokenAndInt(SOFFSET); err != nil {
+		return nil, err
+	}
+
 	return stmt, nil
 }
 
@@ -1050,7 +1200,7 @@ func (p *Parser) parseTagKeys() ([]string, error) {
 
 		// Parse required ) token.
 		if tok, pos, lit = p.scanIgnoreWhitespace(); tok != RPAREN {
-			return nil, newParseError(tokstr(tok, lit), []string{"("}, pos)
+			return nil, newParseError(tokstr(tok, lit), []string{")"}, pos)
 		}
 	} else if tok == EQ {
 		// Parse required tag key.
@@ -1072,6 +1222,13 @@ func (p *Parser) parseShowUsersStatement() (*ShowUsersStatement, error) {
 	return &ShowUsersStatement{}, nil
 }
 
+// parseShowSubscriptionsStatement parses a string and returns a ShowSubscriptionsStatement
+// This function assumes the "SHOW SUBSCRIPTIONS" tokens have been consumed.
+func (p *Parser) parseShowSubscriptionsStatement() (*ShowSubscriptionsStatement, error) {
+	stmt := &ShowSubscriptionsStatement{}
+	return stmt, nil
+}
+
 // parseShowFieldKeysStatement parses a string and returns a ShowSeriesStatement.
 // This function assumes the "SHOW FIELD KEYS" tokens have already been consumed.
 func (p *Parser) parseShowFieldKeysStatement() (*ShowFieldKeysStatement, error) {
@@ -1150,6 +1307,29 @@ func (p *Parser) parseDropSeriesStatement() (*DropSeriesStatement, error) {
 	return stmt, nil
 }
 
+// parseDropServerStatement parses a string and returns a DropServerStatement.
+// This function assumes the "DROP <META|DATA>" tokens have already been consumed.
+func (p *Parser) parseDropServerStatement(tok Token) (*DropServerStatement, error) {
+	// Parse the SERVER token
+	if tok, pos, lit := p.scanIgnoreWhitespace(); tok != SERVER {
+		return nil, newParseError(tokstr(tok, lit), []string{"SERVER"}, pos)
+	}
+
+	s := &DropServerStatement{}
+	var err error
+
+	if tok == META {
+		s.Meta = true
+	}
+
+	// Parse the server's ID.
+	if s.NodeID, err = p.parseUInt64(); err != nil {
+		return nil, err
+	}
+
+	return s, nil
+}
+
 // parseShowContinuousQueriesStatement parses a string and returns a ShowContinuousQueriesStatement.
 // This function assumes the "SHOW CONTINUOUS" tokens have already been consumed.
 func (p *Parser) parseShowContinuousQueriesStatement() (*ShowContinuousQueriesStatement, error) {
@@ -1225,6 +1405,13 @@ func (p *Parser) parseCreateContinuousQueryStatement() (*CreateContinuousQuerySt
 	}
 	stmt.Database = ident
 
+	if p.parseTokenMaybe(RESAMPLE) {
+		stmt.ResampleEvery, stmt.ResampleFor, err = p.parseResample()
+		if err != nil {
+			return nil, err
+		}
+	}
+
 	// Expect a "BEGIN SELECT" tokens.
 	if err := p.parseTokens([]Token{BEGIN, SELECT}); err != nil {
 		return nil, err
@@ -1258,6 +1445,10 @@ func (p *Parser) parseCreateContinuousQueryStatement() (*CreateContinuousQuerySt
 		return nil, newParseError(tokstr(tok, lit), []string{"END"}, pos)
 	}
 
+	if err := stmt.validate(); err != nil {
+		return nil, err
+	}
+
 	return stmt, nil
 }
 
@@ -1283,6 +1474,58 @@ func (p *Parser) parseCreateDatabaseStatement() (*CreateDatabaseStatement, error
 	}
 	stmt.Name = lit
 
+	// Look for "WITH"
+	if tok, _, _ := p.scanIgnoreWhitespace(); tok == WITH {
+		// validate that at least one of DURATION, REPLICATION or NAME is provided
+		tok, pos, lit := p.scanIgnoreWhitespace()
+		if tok != DURATION && tok != REPLICATION && tok != NAME {
+			return nil, newParseError(tokstr(tok, lit), []string{"DURATION", "REPLICATION", "NAME"}, pos)
+		}
+		// rewind
+		p.unscan()
+
+		// mark statement as having a RetentionPolicyInfo defined
+		stmt.RetentionPolicyCreate = true
+
+		// Look for "DURATION"
+		var rpDuration time.Duration // default is forever
+		if err := p.parseTokens([]Token{DURATION}); err != nil {
+			p.unscan()
+		} else {
+			rpDuration, err = p.parseDuration()
+			if err != nil {
+				return nil, err
+			}
+		}
+		stmt.RetentionPolicyDuration = rpDuration
+
+		// Look for "REPLICATION"
+		var rpReplication int = 1 // default is 1
+		if err := p.parseTokens([]Token{REPLICATION}); err != nil {
+			p.unscan()
+		} else {
+			rpReplication, err = p.parseInt(1, math.MaxInt32)
+			if err != nil {
+				return nil, err
+			}
+		}
+		stmt.RetentionPolicyReplication = rpReplication
+
+		// Look for "NAME"
+		var rpName string = "default" // default is default
+		if err := p.parseTokens([]Token{NAME}); err != nil {
+			p.unscan()
+		} else {
+			rpName, err = p.parseIdent()
+			if err != nil {
+				return nil, err
+			}
+		}
+		stmt.RetentionPolicyName = rpName
+	} else {
+		p.unscan()
+	}
+
 	return stmt, nil
 }
 
@@ -1291,6 +1534,16 @@ func (p *Parser) parseCreateDatabaseStatement() (*CreateDatabaseStatement, error
 func (p *Parser) parseDropDatabaseStatement() (*DropDatabaseStatement, error) {
 	stmt := &DropDatabaseStatement{}
 
+	// Look for "IF EXISTS"
+	if tok, _, _ := p.scanIgnoreWhitespace(); tok == IF {
+		if err := p.parseTokens([]Token{EXISTS}); err != nil {
+			return nil, err
+		}
+		stmt.IfExists = true
+	} else {
+		p.unscan()
+	}
+
 	// Parse the name of the database to be dropped.
 	lit, err := p.parseIdent()
 	if err != nil {
@@ -1301,6 +1554,42 @@ func (p *Parser) parseDropDatabaseStatement() (*DropDatabaseStatement, error) {
 	return stmt, nil
 }
 
+// parseDropSubscriptionStatement parses a string and returns a DropSubscriptionStatement.
+// This function assumes the "DROP SUBSCRIPTION" tokens have already been consumed.
+func (p *Parser) parseDropSubscriptionStatement() (*DropSubscriptionStatement, error) {
+	stmt := &DropSubscriptionStatement{}
+
+	// Read the id of the subscription to drop.
+	ident, err := p.parseIdent()
+	if err != nil {
+		return nil, err
+	}
+	stmt.Name = ident
+
+	// Expect an "ON" keyword.
+	if tok, pos, lit := p.scanIgnoreWhitespace(); tok != ON {
+		return nil, newParseError(tokstr(tok, lit), []string{"ON"}, pos)
+	}
+
+	// Read the name of the database.
+	if ident, err = p.parseIdent(); err != nil {
+		return nil, err
+	}
+	stmt.Database = ident
+
+	if tok, pos, lit := p.scan(); tok != DOT {
+		return nil, newParseError(tokstr(tok, lit), []string{"."}, pos)
+	}
+
+	// Read the name of the retention policy.
+	if ident, err = p.parseIdent(); err != nil {
+		return nil, err
+	}
+	stmt.RetentionPolicy = ident
+
+	return stmt, nil
+}
+
 // parseDropRetentionPolicyStatement parses a string and returns a DropRetentionPolicyStatement.
 // This function assumes the DROP RETENTION POLICY tokens have been consumed.
 func (p *Parser) parseDropRetentionPolicyStatement() (*DropRetentionPolicyStatement, error) {
@@ -1411,6 +1700,12 @@ func (p *Parser) parseRetentionPolicy() (name string, dfault bool, err error) {
 	return
 }
 
+// parseShowShardGroupsStatement parses a string for "SHOW SHARD GROUPS" statement.
+// This function assumes the "SHOW SHARD GROUPS" tokens have already been consumed.
+func (p *Parser) parseShowShardGroupsStatement() (*ShowShardGroupsStatement, error) {
+	return &ShowShardGroupsStatement{}, nil
+}
+
 // parseShowShardsStatement parses a string for "SHOW SHARDS" statement.
 // This function assumes the "SHOW SHARDS" tokens have already been consumed.
 func (p *Parser) parseShowShardsStatement() (*ShowShardsStatement, error) {
@@ -1423,8 +1718,8 @@ func (p *Parser) parseShowStatsStatement() (*ShowStatsStatement, error) {
 	stmt := &ShowStatsStatement{}
 	var err error
 
-	if tok, _, _ := p.scanIgnoreWhitespace(); tok == ON {
-		stmt.Host, err = p.parseString()
+	if tok, _, _ := p.scanIgnoreWhitespace(); tok == FOR {
+		stmt.Module, err = p.parseString()
 	} else {
 		p.unscan()
 	}
@@ -1435,7 +1730,15 @@ func (p *Parser) parseShowStatsStatement() (*ShowStatsStatement, error) {
 // parseShowDiagnostics parses a string and returns a ShowDiagnosticsStatement.
 func (p *Parser) parseShowDiagnosticsStatement() (*ShowDiagnosticsStatement, error) {
 	stmt := &ShowDiagnosticsStatement{}
-	return stmt, nil
+	var err error
+
+	if tok, _, _ := p.scanIgnoreWhitespace(); tok == FOR {
+		stmt.Module, err = p.parseString()
+	} else {
+		p.unscan()
+	}
+
+	return stmt, err
 }
 
 // parseDropContinuousQueriesStatement parses a string and returns a DropContinuousQueryStatement.
@@ -1547,7 +1850,7 @@ func (c *validateField) Visit(n Node) Visitor {
 	return c
 }
 
-// parseAlias parses the "AS (IDENT|STRING)" alias for fields and dimensions.
+// parseAlias parses the "AS IDENT" alias for fields and dimensions.
 func (p *Parser) parseAlias() (string, error) {
 	// Check if the next token is "AS". If not, then unscan and exit.
 	if tok, _, _ := p.scanIgnoreWhitespace(); tok != AS {
@@ -2103,6 +2406,47 @@ func (p *Parser) parseCall(name string) (*Call, error) {
 	return &Call{Name: name, Args: args}, nil
 }
 
+// parseResample parses a RESAMPLE [EVERY <duration>] [FOR <duration>].
+// This function assumes RESAMPLE has already been consumed.
+// EVERY and FOR are optional, but at least one of the two has to be used.
+func (p *Parser) parseResample() (time.Duration, time.Duration, error) {
+	var interval time.Duration
+	if p.parseTokenMaybe(EVERY) {
+		tok, pos, lit := p.scanIgnoreWhitespace()
+		if tok != DURATION_VAL {
+			return 0, 0, newParseError(tokstr(tok, lit), []string{"duration"}, pos)
+		}
+
+		d, err := ParseDuration(lit)
+		if err != nil {
+			return 0, 0, &ParseError{Message: err.Error(), Pos: pos}
+		}
+		interval = d
+	}
+
+	var maxDuration time.Duration
+	if p.parseTokenMaybe(FOR) {
+		tok, pos, lit := p.scanIgnoreWhitespace()
+		if tok != DURATION_VAL {
+			return 0, 0, newParseError(tokstr(tok, lit), []string{"duration"}, pos)
+		}
+
+		d, err := ParseDuration(lit)
+		if err != nil {
+			return 0, 0, &ParseError{Message: err.Error(), Pos: pos}
+		}
+		maxDuration = d
+	}
+
+	// Neither EVERY or FOR were read, so read the next token again
+	// so we can return a suitable error message.
+	if interval == 0 && maxDuration == 0 {
+		tok, pos, lit := p.scanIgnoreWhitespace()
+		return 0, 0, newParseError(tokstr(tok, lit), []string{"EVERY", "FOR"}, pos)
+	}
+	return interval, maxDuration, nil
+}
+
 // scan returns the next token from the underlying scanner.
 func (p *Parser) scan() (tok Token, pos Pos, lit string) { return p.s.Scan() }
 
@@ -2127,16 +2471,8 @@ func (p *Parser) unscan() { p.s.Unscan() }
 
 // ParseDuration parses a time duration from a string.
 func ParseDuration(s string) (time.Duration, error) {
-	// Return an error if the string is blank.
-	if len(s) == 0 {
-		return 0, ErrInvalidDuration
-	}
-
-	// If there's only character then it must be a digit (in microseconds).
-	if len(s) == 1 {
-		if n, err := strconv.ParseInt(s, 10, 64); err == nil {
-			return time.Duration(n) * time.Microsecond, nil
-		}
+	// Return an error if the string is blank or one character
+	if len(s) < 2 {
 		return 0, ErrInvalidDuration
 	}
 
@@ -2144,13 +2480,10 @@ func ParseDuration(s string) (time.Duration, error) {
 	a := split(s)
 
 	// Extract the unit of measure.
-	// If the last character is a digit then parse the whole string as microseconds.
-	// If the last two characters are "ms" the parse as milliseconds.
+	// If the last two characters are "ms" then parse as milliseconds.
 	// Otherwise just use the last character as the unit of measure.
 	var num, uom string
-	if isDigit(rune(a[len(a)-1])) {
-		num, uom = s, "u"
-	} else if len(s) > 2 && s[len(s)-2:] == "ms" {
+	if len(s) > 2 && s[len(s)-2:] == "ms" {
 		num, uom = string(a[:len(a)-2]), "ms"
 	} else {
 		num, uom = string(a[:len(a)-1]), string(a[len(a)-1:])
@@ -2200,7 +2533,10 @@ func FormatDuration(d time.Duration) string {
 	} else if d%time.Millisecond == 0 {
 		return fmt.Sprintf("%dms", d/time.Millisecond)
 	}
-	return fmt.Sprintf("%d", d/time.Microsecond)
+	// Although we accept both "u" and "µ" when reading microsecond durations,
+	// we output with "u", which can be represented in 1 byte,
+	// instead of "µ", which requires 2 bytes.
+	return fmt.Sprintf("%du", d/time.Microsecond)
 }
 
 // parseTokens consumes an expected sequence of tokens.
@@ -2213,6 +2549,17 @@ func (p *Parser) parseTokens(toks []Token) error {
 	return nil
 }
 
+// parseTokenMaybe consumes the next token if it matches the expected one and
+// does nothing if the next token is not the next one.
+func (p *Parser) parseTokenMaybe(expected Token) bool {
+	tok, _, _ := p.scanIgnoreWhitespace()
+	if tok != expected {
+		p.unscan()
+		return false
+	}
+	return true
+}
+
 // QuoteString returns a quoted string.
 func QuoteString(s string) string {
 	return `'` + strings.NewReplacer("\n", `\n`, `\`, `\\`, `'`, `\'`).Replace(s) + `'`
