@@ -11,13 +11,20 @@ import (
 	"sync"
 	"time"
 
+	"github.com/influxdb/influxdb"
 	"github.com/influxdb/influxdb/cluster"
-	"github.com/influxdb/influxdb/meta"
-	"github.com/influxdb/influxdb/tsdb"
+	"github.com/influxdb/influxdb/models"
+	"github.com/influxdb/influxdb/services/meta"
 )
 
 const leaderWaitTimeout = 30 * time.Second
 
+// Policy constants.
+const (
+	MonitorRetentionPolicy         = "monitor"
+	MonitorRetentionPolicyDuration = 7 * 24 * time.Hour
+)
+
 // DiagsClient is the interface modules implement if they register diags with monitor.
 type DiagsClient interface {
 	Diagnostics() (*Diagnostic, error)
@@ -45,6 +52,7 @@ type Diagnostic struct {
 	Rows    [][]interface{}
 }
 
+// NewDiagnostic initialises a new Diagnostic with the specified columns.
 func NewDiagnostic(columns []string) *Diagnostic {
 	return &Diagnostic{
 		Columns: columns,
@@ -52,6 +60,7 @@ func NewDiagnostic(columns []string) *Diagnostic {
 	}
 }
 
+// AddRow appends the provided row to the Diagnostic's rows.
 func (d *Diagnostic) AddRow(r []interface{}) {
 	d.Rows = append(d.Rows, r)
 }
@@ -59,9 +68,10 @@ func (d *Diagnostic) AddRow(r []interface{}) {
 // Monitor represents an instance of the monitor system.
 type Monitor struct {
 	// Build information for diagnostics.
-	Version string
-	Commit  string
-	Branch  string
+	Version   string
+	Commit    string
+	Branch    string
+	BuildTime string
 
 	wg   sync.WaitGroup
 	done chan struct{}
@@ -69,6 +79,7 @@ type Monitor struct {
 
 	diagRegistrations map[string]DiagsClient
 
+	storeCreated           bool
 	storeEnabled           bool
 	storeDatabase          string
 	storeRetentionPolicy   string
@@ -77,13 +88,16 @@ type Monitor struct {
 	storeAddress           string
 	storeInterval          time.Duration
 
-	MetaStore interface {
-		ClusterID() (uint64, error)
-		NodeID() uint64
-		WaitForLeader(d time.Duration) error
-		CreateDatabaseIfNotExists(name string) (*meta.DatabaseInfo, error)
+	MetaClient interface {
+		ClusterID() uint64
+		CreateDatabase(name string) (*meta.DatabaseInfo, error)
+		CreateRetentionPolicy(database string, rpi *meta.RetentionPolicyInfo) (*meta.RetentionPolicyInfo, error)
+		SetDefaultRetentionPolicy(database, name string) error
+		DropRetentionPolicy(database, name string) error
 	}
 
+	NodeID uint64
+
 	PointsWriter interface {
 		WritePoints(p *cluster.WritePointsRequest) error
 	}
@@ -104,7 +118,7 @@ func New(c Config) *Monitor {
 }
 
 // Open opens the monitoring system, using the given clusterID, node ID, and hostname
-// for identification purposem.
+// for identification purpose.
 func (m *Monitor) Open() error {
 	m.Logger.Printf("Starting monitor system")
 
@@ -113,6 +127,7 @@ func (m *Monitor) Open() error {
 		Version: m.Version,
 		Commit:  m.Commit,
 		Branch:  m.Branch,
+		Time:    m.BuildTime,
 	})
 	m.RegisterDiagnosticsClient("runtime", &goRuntime{})
 	m.RegisterDiagnosticsClient("network", &network{})
@@ -143,18 +158,24 @@ func (m *Monitor) SetLogger(l *log.Logger) {
 }
 
 // RegisterDiagnosticsClient registers a diagnostics client with the given name and tags.
-func (m *Monitor) RegisterDiagnosticsClient(name string, client DiagsClient) error {
+func (m *Monitor) RegisterDiagnosticsClient(name string, client DiagsClient) {
 	m.mu.Lock()
 	defer m.mu.Unlock()
 	m.diagRegistrations[name] = client
 	m.Logger.Printf(`'%s' registered for diagnostics monitoring`, name)
-	return nil
+}
+
+// DeregisterDiagnosticsClient deregisters a diagnostics client by name.
+func (m *Monitor) DeregisterDiagnosticsClient(name string) {
+	m.mu.Lock()
+	defer m.mu.Unlock()
+	delete(m.diagRegistrations, name)
 }
 
 // Statistics returns the combined statistics for all expvar data. The given
 // tags are added to each of the returned statistics.
-func (m *Monitor) Statistics(tags map[string]string) ([]*statistic, error) {
-	statistics := make([]*statistic, 0)
+func (m *Monitor) Statistics(tags map[string]string) ([]*Statistic, error) {
+	var statistics []*Statistic
 
 	expvar.Do(func(kv expvar.KeyValue) {
 		// Skip built-in expvar stats.
@@ -162,7 +183,7 @@ func (m *Monitor) Statistics(tags map[string]string) ([]*statistic, error) {
 			return
 		}
 
-		statistic := &statistic{
+		statistic := &Statistic{
 			Tags:   make(map[string]string),
 			Values: make(map[string]interface{}),
 		}
@@ -228,11 +249,17 @@ func (m *Monitor) Statistics(tags map[string]string) ([]*statistic, error) {
 	})
 
 	// Add Go memstats.
-	statistic := &statistic{
+	statistic := &Statistic{
 		Name:   "runtime",
 		Tags:   make(map[string]string),
 		Values: make(map[string]interface{}),
 	}
+
+	// Add any supplied tags to Go memstats
+	for k, v := range tags {
+		statistic.Tags[k] = v
+	}
+
 	var rt runtime.MemStats
 	runtime.ReadMemStats(&rt)
 	statistic.Values = map[string]interface{}{
@@ -257,6 +284,9 @@ func (m *Monitor) Statistics(tags map[string]string) ([]*statistic, error) {
 	return statistics, nil
 }
 
+// Diagnostics fetches diagnostic information for each registered
+// diagnostic client. It skips any clients that return an error when
+// retrieving their diagnostics.
 func (m *Monitor) Diagnostics() (map[string]*Diagnostic, error) {
 	m.mu.Lock()
 	defer m.mu.Unlock()
@@ -272,47 +302,79 @@ func (m *Monitor) Diagnostics() (map[string]*Diagnostic, error) {
 	return diags, nil
 }
 
+// createInternalStorage ensures the internal storage has been created.
+func (m *Monitor) createInternalStorage() {
+	if m.storeCreated {
+		return
+	}
+
+	if _, err := m.MetaClient.CreateDatabase(m.storeDatabase); err != nil {
+		m.Logger.Printf("failed to create database '%s', failed to create storage: %s",
+			m.storeDatabase, err.Error())
+		return
+	}
+
+	rpi := meta.NewRetentionPolicyInfo(MonitorRetentionPolicy)
+	rpi.Duration = MonitorRetentionPolicyDuration
+	rpi.ReplicaN = 1
+	if _, err := m.MetaClient.CreateRetentionPolicy(m.storeDatabase, rpi); err != nil {
+		m.Logger.Printf("failed to create retention policy '%s', failed to create internal storage: %s",
+			rpi.Name, err.Error())
+		return
+	}
+
+	if err := m.MetaClient.SetDefaultRetentionPolicy(m.storeDatabase, rpi.Name); err != nil {
+		m.Logger.Printf("failed to set default retention policy on '%s', failed to create internal storage: %s",
+			m.storeDatabase, err.Error())
+		return
+	}
+
+	err := m.MetaClient.DropRetentionPolicy(m.storeDatabase, "default")
+	if err != nil && err.Error() != influxdb.ErrRetentionPolicyNotFound("default").Error() {
+		m.Logger.Printf("failed to delete retention policy 'default', failed to created internal storage: %s", err.Error())
+		return
+	}
+
+	// Mark storage creation complete.
+	m.storeCreated = true
+}
+
 // storeStatistics writes the statistics to an InfluxDB system.
 func (m *Monitor) storeStatistics() {
 	defer m.wg.Done()
 	m.Logger.Printf("Storing statistics in database '%s' retention policy '%s', at interval %s",
 		m.storeDatabase, m.storeRetentionPolicy, m.storeInterval)
 
-	if err := m.MetaStore.WaitForLeader(leaderWaitTimeout); err != nil {
-		m.Logger.Printf("failed to detect a cluster leader, terminating storage: %s", err.Error())
-		return
-	}
-
 	// Get cluster-level metadata. Nothing different is going to happen if errors occur.
-	clusterID, _ := m.MetaStore.ClusterID()
-	nodeID := m.MetaStore.NodeID()
+	clusterID := m.MetaClient.ClusterID()
 	hostname, _ := os.Hostname()
 	clusterTags := map[string]string{
 		"clusterID": fmt.Sprintf("%d", clusterID),
-		"nodeID":    fmt.Sprintf("%d", nodeID),
+		"nodeID":    fmt.Sprintf("%d", m.NodeID),
 		"hostname":  hostname,
 	}
 
-	if _, err := m.MetaStore.CreateDatabaseIfNotExists(m.storeDatabase); err != nil {
-		m.Logger.Printf("failed to create database '%s', terminating storage: %s",
-			m.storeDatabase, err.Error())
-		return
-	}
-
 	tick := time.NewTicker(m.storeInterval)
 	defer tick.Stop()
 	for {
 		select {
 		case <-tick.C:
+			m.createInternalStorage()
+
 			stats, err := m.Statistics(clusterTags)
 			if err != nil {
 				m.Logger.Printf("failed to retrieve registered statistics: %s", err)
 				continue
 			}
 
-			points := make(tsdb.Points, 0, len(stats))
+			points := make(models.Points, 0, len(stats))
 			for _, s := range stats {
-				points = append(points, tsdb.NewPoint(s.Name, s.Tags, s.Values, time.Now()))
+				pt, err := models.NewPoint(s.Name, s.Tags, s.Values, time.Now().Truncate(time.Second))
+				if err != nil {
+					m.Logger.Printf("Dropping point %v: %v", s.Name, err)
+					continue
+				}
+				points = append(points, pt)
 			}
 
 			err = m.PointsWriter.WritePoints(&cluster.WritePointsRequest{
@@ -332,16 +394,16 @@ func (m *Monitor) storeStatistics() {
 	}
 }
 
-// statistic represents the information returned by a single monitor client.
-type statistic struct {
-	Name   string
-	Tags   map[string]string
-	Values map[string]interface{}
+// Statistic represents the information returned by a single monitor client.
+type Statistic struct {
+	Name   string                 `json:"name"`
+	Tags   map[string]string      `json:"tags"`
+	Values map[string]interface{} `json:"values"`
 }
 
 // newStatistic returns a new statistic object.
-func newStatistic(name string, tags map[string]string, values map[string]interface{}) *statistic {
-	return &statistic{
+func newStatistic(name string, tags map[string]string, values map[string]interface{}) *Statistic {
+	return &Statistic{
 		Name:   name,
 		Tags:   tags,
 		Values: values,
@@ -349,9 +411,9 @@ func newStatistic(name string, tags map[string]string, values map[string]interfa
 }
 
 // valueNames returns a sorted list of the value names, if any.
-func (s *statistic) valueNames() []string {
+func (s *Statistic) valueNames() []string {
 	a := make([]string, 0, len(s.Values))
-	for k, _ := range s.Values {
+	for k := range s.Values {
 		a = append(a, k)
 	}
 	sort.Strings(a)
@@ -362,7 +424,7 @@ func (s *statistic) valueNames() []string {
 func DiagnosticFromMap(m map[string]interface{}) *Diagnostic {
 	// Display columns in deterministic order.
 	sortedKeys := make([]string, 0, len(m))
-	for k, _ := range m {
+	for k := range m {
 		sortedKeys = append(sortedKeys, k)
 	}
 	sort.Strings(sortedKeys)
