@@ -1,13 +1,13 @@
 package graphite_test
 
 import (
-	"math"
+	"reflect"
 	"strconv"
 	"testing"
 	"time"
 
+	"github.com/influxdb/influxdb/models"
 	"github.com/influxdb/influxdb/services/graphite"
-	"github.com/influxdb/influxdb/tsdb"
 )
 
 func BenchmarkParse(b *testing.B) {
@@ -105,7 +105,7 @@ func TestTemplateApply(t *testing.T) {
 			continue
 		}
 
-		measurement, tags := tmpl.Apply(test.input)
+		measurement, tags, _, _ := tmpl.Apply(test.input)
 		if measurement != test.measurement {
 			t.Fatalf("name parse failer.  expected %v, got %v", test.measurement, measurement)
 		}
@@ -224,22 +224,13 @@ func TestParseNaN(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	pt, err := p.Parse("servers.localhost.cpu_load NaN 1435077219")
-	if err != nil {
-		t.Fatalf("parse error: %v", err)
+	_, err = p.Parse("servers.localhost.cpu_load NaN 1435077219")
+	if err == nil {
+		t.Fatalf("expected error. got nil")
 	}
 
-	exp := tsdb.NewPoint("servers.localhost.cpu_load",
-		tsdb.Tags{},
-		tsdb.Fields{"value": math.NaN()},
-		time.Unix(1435077219, 0))
-
-	if exp.String() != pt.String() {
-		t.Errorf("parse mismatch: got %v, exp %v", pt.String(), exp.String())
-	}
-
-	if !math.IsNaN(pt.Fields()["value"].(float64)) {
-		t.Errorf("parse value mismatch: expected NaN")
+	if _, ok := err.(*graphite.UnsupposedValueError); !ok {
+		t.Fatalf("expected *graphite.ErrUnsupportedValue, got %v", reflect.TypeOf(err))
 	}
 }
 
@@ -249,9 +240,9 @@ func TestFilterMatchDefault(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("miss.servers.localhost.cpu_load",
-		tsdb.Tags{},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("miss.servers.localhost.cpu_load",
+		models.Tags{},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("miss.servers.localhost.cpu_load 11 1435077219")
@@ -270,9 +261,9 @@ func TestFilterMatchMultipleMeasurement(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("cpu.cpu_load.10",
-		tsdb.Tags{"host": "localhost"},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("cpu.cpu_load.10",
+		models.Tags{"host": "localhost"},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.localhost.cpu.cpu_load.10 11 1435077219")
@@ -294,9 +285,9 @@ func TestFilterMatchMultipleMeasurementSeparator(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("cpu_cpu_load_10",
-		tsdb.Tags{"host": "localhost"},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("cpu_cpu_load_10",
+		models.Tags{"host": "localhost"},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.localhost.cpu.cpu_load.10 11 1435077219")
@@ -315,9 +306,9 @@ func TestFilterMatchSingle(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("cpu_load",
-		tsdb.Tags{"host": "localhost"},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("cpu_load",
+		models.Tags{"host": "localhost"},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.localhost.cpu_load 11 1435077219")
@@ -336,9 +327,9 @@ func TestParseNoMatch(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("servers.localhost.memory.VmallocChunk",
-		tsdb.Tags{},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("servers.localhost.memory.VmallocChunk",
+		models.Tags{},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.localhost.memory.VmallocChunk 11 1435077219")
@@ -357,9 +348,9 @@ func TestFilterMatchWildcard(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("cpu_load",
-		tsdb.Tags{"host": "localhost"},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("cpu_load",
+		models.Tags{"host": "localhost"},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.localhost.cpu_load 11 1435077219")
@@ -380,9 +371,9 @@ func TestFilterMatchExactBeforeWildcard(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("cpu_load",
-		tsdb.Tags{"host": "localhost"},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("cpu_load",
+		models.Tags{"host": "localhost"},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.localhost.cpu_load 11 1435077219")
@@ -408,9 +399,9 @@ func TestFilterMatchMostLongestFilter(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("cpu_load",
-		tsdb.Tags{"host": "localhost", "resource": "cpu"},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("cpu_load",
+		models.Tags{"host": "localhost", "resource": "cpu"},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.localhost.cpu.cpu_load 11 1435077219")
@@ -435,9 +426,9 @@ func TestFilterMatchMultipleWildcards(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("cpu_load",
-		tsdb.Tags{"host": "server01"},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("cpu_load",
+		models.Tags{"host": "server01"},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.server01.cpu_load 11 1435077219")
@@ -451,7 +442,7 @@ func TestFilterMatchMultipleWildcards(t *testing.T) {
 }
 
 func TestParseDefaultTags(t *testing.T) {
-	p, err := graphite.NewParser([]string{"servers.localhost .host.measurement*"}, tsdb.Tags{
+	p, err := graphite.NewParser([]string{"servers.localhost .host.measurement*"}, models.Tags{
 		"region": "us-east",
 		"zone":   "1c",
 		"host":   "should not set",
@@ -460,9 +451,9 @@ func TestParseDefaultTags(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("cpu_load",
-		tsdb.Tags{"host": "localhost", "region": "us-east", "zone": "1c"},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("cpu_load",
+		models.Tags{"host": "localhost", "region": "us-east", "zone": "1c"},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.localhost.cpu_load 11 1435077219")
@@ -476,7 +467,7 @@ func TestParseDefaultTags(t *testing.T) {
 }
 
 func TestParseDefaultTemplateTags(t *testing.T) {
-	p, err := graphite.NewParser([]string{"servers.localhost .host.measurement* zone=1c"}, tsdb.Tags{
+	p, err := graphite.NewParser([]string{"servers.localhost .host.measurement* zone=1c"}, models.Tags{
 		"region": "us-east",
 		"host":   "should not set",
 	})
@@ -484,9 +475,9 @@ func TestParseDefaultTemplateTags(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("cpu_load",
-		tsdb.Tags{"host": "localhost", "region": "us-east", "zone": "1c"},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("cpu_load",
+		models.Tags{"host": "localhost", "region": "us-east", "zone": "1c"},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.localhost.cpu_load 11 1435077219")
@@ -500,7 +491,7 @@ func TestParseDefaultTemplateTags(t *testing.T) {
 }
 
 func TestParseDefaultTemplateTagsOverridGlobal(t *testing.T) {
-	p, err := graphite.NewParser([]string{"servers.localhost .host.measurement* zone=1c,region=us-east"}, tsdb.Tags{
+	p, err := graphite.NewParser([]string{"servers.localhost .host.measurement* zone=1c,region=us-east"}, models.Tags{
 		"region": "shot not be set",
 		"host":   "should not set",
 	})
@@ -508,9 +499,9 @@ func TestParseDefaultTemplateTagsOverridGlobal(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("cpu_load",
-		tsdb.Tags{"host": "localhost", "region": "us-east", "zone": "1c"},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("cpu_load",
+		models.Tags{"host": "localhost", "region": "us-east", "zone": "1c"},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.localhost.cpu_load 11 1435077219")
@@ -524,7 +515,7 @@ func TestParseDefaultTemplateTagsOverridGlobal(t *testing.T) {
 }
 
 func TestParseTemplateWhitespace(t *testing.T) {
-	p, err := graphite.NewParser([]string{"servers.localhost        .host.measurement*           zone=1c"}, tsdb.Tags{
+	p, err := graphite.NewParser([]string{"servers.localhost        .host.measurement*           zone=1c"}, models.Tags{
 		"region": "us-east",
 		"host":   "should not set",
 	})
@@ -532,9 +523,9 @@ func TestParseTemplateWhitespace(t *testing.T) {
 		t.Fatalf("unexpected error creating parser, got %v", err)
 	}
 
-	exp := tsdb.NewPoint("cpu_load",
-		tsdb.Tags{"host": "localhost", "region": "us-east", "zone": "1c"},
-		tsdb.Fields{"value": float64(11)},
+	exp := models.MustNewPoint("cpu_load",
+		models.Tags{"host": "localhost", "region": "us-east", "zone": "1c"},
+		models.Fields{"value": float64(11)},
 		time.Unix(1435077219, 0))
 
 	pt, err := p.Parse("servers.localhost.cpu_load 11 1435077219")
@@ -546,3 +537,132 @@ func TestParseTemplateWhitespace(t *testing.T) {
 		t.Errorf("parse mismatch: got %v, exp %v", pt.String(), exp.String())
 	}
 }
+
+// Test basic functionality of ApplyTemplate
+func TestApplyTemplate(t *testing.T) {
+	o := graphite.Options{
+		Separator: "_",
+		Templates: []string{"current.* measurement.measurement"},
+	}
+	p, err := graphite.NewParserWithOptions(o)
+	if err != nil {
+		t.Fatalf("unexpected error creating parser, got %v", err)
+	}
+
+	measurement, _, _, _ := p.ApplyTemplate("current.users")
+	if measurement != "current_users" {
+		t.Errorf("Parser.ApplyTemplate unexpected result. got %s, exp %s",
+			measurement, "current_users")
+	}
+}
+
+// Test basic functionality of ApplyTemplate
+func TestApplyTemplateNoMatch(t *testing.T) {
+	o := graphite.Options{
+		Separator: "_",
+		Templates: []string{"foo.bar measurement.measurement"},
+	}
+	p, err := graphite.NewParserWithOptions(o)
+	if err != nil {
+		t.Fatalf("unexpected error creating parser, got %v", err)
+	}
+
+	measurement, _, _, _ := p.ApplyTemplate("current.users")
+	if measurement != "current.users" {
+		t.Errorf("Parser.ApplyTemplate unexpected result. got %s, exp %s",
+			measurement, "current.users")
+	}
+}
+
+// Test that most specific template is chosen
+func TestApplyTemplateSpecific(t *testing.T) {
+	o := graphite.Options{
+		Separator: "_",
+		Templates: []string{
+			"current.* measurement.measurement",
+			"current.*.* measurement.measurement.service",
+		},
+	}
+	p, err := graphite.NewParserWithOptions(o)
+	if err != nil {
+		t.Fatalf("unexpected error creating parser, got %v", err)
+	}
+
+	measurement, tags, _, _ := p.ApplyTemplate("current.users.facebook")
+	if measurement != "current_users" {
+		t.Errorf("Parser.ApplyTemplate unexpected result. got %s, exp %s",
+			measurement, "current_users")
+	}
+	service, ok := tags["service"]
+	if !ok {
+		t.Error("Expected for template to apply a 'service' tag, but not found")
+	}
+	if service != "facebook" {
+		t.Errorf("Expected service='facebook' tag, got service='%s'", service)
+	}
+}
+
+func TestApplyTemplateTags(t *testing.T) {
+	o := graphite.Options{
+		Separator: "_",
+		Templates: []string{"current.* measurement.measurement region=us-west"},
+	}
+	p, err := graphite.NewParserWithOptions(o)
+	if err != nil {
+		t.Fatalf("unexpected error creating parser, got %v", err)
+	}
+
+	measurement, tags, _, _ := p.ApplyTemplate("current.users")
+	if measurement != "current_users" {
+		t.Errorf("Parser.ApplyTemplate unexpected result. got %s, exp %s",
+			measurement, "current_users")
+	}
+
+	region, ok := tags["region"]
+	if !ok {
+		t.Error("Expected for template to apply a 'region' tag, but not found")
+	}
+	if region != "us-west" {
+		t.Errorf("Expected region='us-west' tag, got region='%s'", region)
+	}
+}
+
+func TestApplyTemplateField(t *testing.T) {
+	o := graphite.Options{
+		Separator: "_",
+		Templates: []string{"current.* measurement.measurement.field"},
+	}
+	p, err := graphite.NewParserWithOptions(o)
+	if err != nil {
+		t.Fatalf("unexpected error creating parser, got %v", err)
+	}
+
+	measurement, _, field, err := p.ApplyTemplate("current.users.logged_in")
+
+	if measurement != "current_users" {
+		t.Errorf("Parser.ApplyTemplate unexpected result. got %s, exp %s",
+			measurement, "current_users")
+	}
+
+	if field != "logged_in" {
+		t.Errorf("Parser.ApplyTemplate unexpected result. got %s, exp %s",
+			field, "logged_in")
+	}
+}
+
+func TestApplyTemplateFieldError(t *testing.T) {
+	o := graphite.Options{
+		Separator: "_",
+		Templates: []string{"current.* measurement.field.field"},
+	}
+	p, err := graphite.NewParserWithOptions(o)
+	if err != nil {
+		t.Fatalf("unexpected error creating parser, got %v", err)
+	}
+
+	_, _, _, err = p.ApplyTemplate("current.users.logged_in")
+	if err == nil {
+		t.Errorf("Parser.ApplyTemplate unexpected result. got %s, exp %s", err,
+			"'field' can only be used once in each template: current.users.logged_in")
+	}
+}
