@@ -14,8 +14,8 @@ import (
 
 	"github.com/influxdb/influxdb"
 	"github.com/influxdb/influxdb/cluster"
-	"github.com/influxdb/influxdb/meta"
 	"github.com/influxdb/influxdb/monitor"
+	"github.com/influxdb/influxdb/services/meta"
 	"github.com/influxdb/influxdb/tsdb"
 )
 
@@ -24,65 +24,33 @@ const (
 	leaderWaitTimeout = 30 * time.Second
 )
 
-// Initialize the graphite stats and diags
-func init() {
-	tcpConnections = make(map[string]*tcpConnectionDiag)
-}
+// statistics gathered by the graphite package.
+const (
+	statPointsReceived      = "pointsRx"
+	statBytesReceived       = "bytesRx"
+	statPointsParseFail     = "pointsParseFail"
+	statPointsNaNFail       = "pointsNaNFail"
+	statPointsUnsupported   = "pointsUnsupportedFail"
+	statBatchesTransmitted  = "batchesTx"
+	statPointsTransmitted   = "pointsTx"
+	statBatchesTransmitFail = "batchesTxFail"
+	statConnectionsActive   = "connsActive"
+	statConnectionsHandled  = "connsHandled"
+)
 
-// Package-level tracking of connections for diagnostics.
-var monitorOnce sync.Once
-
-type tcpConnectionDiag struct {
+type tcpConnection struct {
 	conn        net.Conn
 	connectTime time.Time
 }
 
-var tcpConnectionsMu sync.Mutex
-var tcpConnections map[string]*tcpConnectionDiag
-
-func addConnection(c net.Conn) {
-	tcpConnectionsMu.Lock()
-	defer tcpConnectionsMu.Unlock()
-	tcpConnections[c.RemoteAddr().String()] = &tcpConnectionDiag{
-		conn:        c,
-		connectTime: time.Now().UTC(),
-	}
-}
-func removeConnection(c net.Conn) {
-	tcpConnectionsMu.Lock()
-	defer tcpConnectionsMu.Unlock()
-	delete(tcpConnections, c.RemoteAddr().String())
+func (c *tcpConnection) Close() {
+	c.conn.Close()
 }
 
-func handleDiagnostics() (*monitor.Diagnostic, error) {
-	tcpConnectionsMu.Lock()
-	defer tcpConnectionsMu.Unlock()
-
-	d := &monitor.Diagnostic{
-		Columns: []string{"local", "remote", "connect time"},
-		Rows:    make([][]interface{}, 0, len(tcpConnections)),
-	}
-	for _, v := range tcpConnections {
-		_ = v
-		d.Rows = append(d.Rows, []interface{}{v.conn.LocalAddr().String(), v.conn.RemoteAddr().String(), v.connectTime})
-	}
-	return d, nil
-}
-
-// statistics gathered by the graphite package.
-const (
-	statPointsReceived      = "points_rx"
-	statBytesReceived       = "bytes_rx"
-	statPointsParseFail     = "points_parse_fail"
-	statPointsUnsupported   = "points_unsupported_fail"
-	statBatchesTrasmitted   = "batches_tx"
-	statPointsTransmitted   = "points_tx"
-	statBatchesTransmitFail = "batches_tx_fail"
-	statConnectionsActive   = "connections_active"
-	statConnectionsHandled  = "connections_handled"
-)
-
+// Service represents a Graphite service.
 type Service struct {
+	mu sync.Mutex
+
 	bindAddress      string
 	database         string
 	protocol         string
@@ -90,12 +58,15 @@ type Service struct {
 	batchPending     int
 	batchTimeout     time.Duration
 	consistencyLevel cluster.ConsistencyLevel
+	udpReadBuffer    int
 
 	batcher *tsdb.PointBatcher
 	parser  *Parser
 
-	logger  *log.Logger
-	statMap *expvar.Map
+	logger           *log.Logger
+	statMap          *expvar.Map
+	tcpConnectionsMu sync.Mutex
+	tcpConnections   map[string]*tcpConnection
 
 	ln      net.Listener
 	addr    net.Addr
@@ -105,14 +76,14 @@ type Service struct {
 	done chan struct{}
 
 	Monitor interface {
-		RegisterDiagnosticsClient(name string, client monitor.DiagsClient) error
+		RegisterDiagnosticsClient(name string, client monitor.DiagsClient)
+		DeregisterDiagnosticsClient(name string)
 	}
 	PointsWriter interface {
 		WritePoints(p *cluster.WritePointsRequest) error
 	}
-	MetaStore interface {
-		WaitForLeader(d time.Duration) error
-		CreateDatabaseIfNotExists(name string) (*meta.DatabaseInfo, error)
+	MetaClient interface {
+		CreateDatabase(name string) (*meta.DatabaseInfo, error)
 	}
 }
 
@@ -122,14 +93,16 @@ func NewService(c Config) (*Service, error) {
 	d := c.WithDefaults()
 
 	s := Service{
-		bindAddress:  d.BindAddress,
-		database:     d.Database,
-		protocol:     d.Protocol,
-		batchSize:    d.BatchSize,
-		batchPending: d.BatchPending,
-		batchTimeout: time.Duration(d.BatchTimeout),
-		logger:       log.New(os.Stderr, "[graphite] ", log.LstdFlags),
-		done:         make(chan struct{}),
+		bindAddress:    d.BindAddress,
+		database:       d.Database,
+		protocol:       d.Protocol,
+		batchSize:      d.BatchSize,
+		batchPending:   d.BatchPending,
+		udpReadBuffer:  d.UDPReadBuffer,
+		batchTimeout:   time.Duration(d.BatchTimeout),
+		logger:         log.New(os.Stderr, "[graphite] ", log.LstdFlags),
+		tcpConnections: make(map[string]*tcpConnection),
+		done:           make(chan struct{}),
 	}
 
 	consistencyLevel, err := cluster.ParseConsistencyLevel(d.ConsistencyLevel)
@@ -153,6 +126,9 @@ func NewService(c Config) (*Service, error) {
 
 // Open starts the Graphite input processing data.
 func (s *Service) Open() error {
+	s.mu.Lock()
+	defer s.mu.Unlock()
+
 	s.logger.Printf("Starting graphite service, batch size %d, batch timeout %s", s.batchSize, s.batchTimeout)
 
 	// Configure expvar monitoring. It's OK to do this even if the service fails to open and
@@ -161,21 +137,12 @@ func (s *Service) Open() error {
 	tags := map[string]string{"proto": s.protocol, "bind": s.bindAddress}
 	s.statMap = influxdb.NewStatistics(key, "graphite", tags)
 
-	// One Graphite service hooks up diagnostics for all Graphite functionality.
-	monitorOnce.Do(func() {
-		if s.Monitor == nil {
-			s.logger.Println("no monitor service available, no monitoring will be performed")
-			return
-		}
-		s.Monitor.RegisterDiagnosticsClient("graphite", monitor.DiagsClientFunc(handleDiagnostics))
-	})
-
-	if err := s.MetaStore.WaitForLeader(leaderWaitTimeout); err != nil {
-		s.logger.Printf("Failed to detect a cluster leader: %s", err.Error())
-		return err
+	// Register diagnostics if a Monitor service is available.
+	if s.Monitor != nil {
+		s.Monitor.RegisterDiagnosticsClient(key, s)
 	}
 
-	if _, err := s.MetaStore.CreateDatabaseIfNotExists(s.database); err != nil {
+	if _, err := s.MetaClient.CreateDatabase(s.database); err != nil {
 		s.logger.Printf("Failed to ensure target database %s exists: %s", s.database, err.Error())
 		return err
 	}
@@ -202,9 +169,21 @@ func (s *Service) Open() error {
 	s.logger.Printf("Listening on %s: %s", strings.ToUpper(s.protocol), s.addr.String())
 	return nil
 }
+func (s *Service) closeAllConnections() {
+	s.tcpConnectionsMu.Lock()
+	defer s.tcpConnectionsMu.Unlock()
+	for _, c := range s.tcpConnections {
+		c.Close()
+	}
+}
 
 // Close stops all data processing on the Graphite input.
 func (s *Service) Close() error {
+	s.mu.Lock()
+	defer s.mu.Unlock()
+
+	s.closeAllConnections()
+
 	if s.ln != nil {
 		s.ln.Close()
 	}
@@ -212,7 +191,9 @@ func (s *Service) Close() error {
 		s.udpConn.Close()
 	}
 
-	s.batcher.Stop()
+	if s.batcher != nil {
+		s.batcher.Stop()
+	}
 	close(s.done)
 	s.wg.Wait()
 	s.done = nil
@@ -225,6 +206,7 @@ func (s *Service) SetLogger(l *log.Logger) {
 	s.logger = l
 }
 
+// Addr returns the address the Service binds to.
 func (s *Service) Addr() net.Addr {
 	return s.addr
 }
@@ -262,11 +244,11 @@ func (s *Service) openTCPServer() (net.Addr, error) {
 func (s *Service) handleTCPConnection(conn net.Conn) {
 	defer s.wg.Done()
 	defer conn.Close()
-	defer removeConnection(conn)
 	defer s.statMap.Add(statConnectionsActive, -1)
-	addConnection(conn)
+	defer s.untrackConnection(conn)
 	s.statMap.Add(statConnectionsActive, 1)
 	s.statMap.Add(statConnectionsHandled, 1)
+	s.trackConnection(conn)
 
 	reader := bufio.NewReader(conn)
 
@@ -286,6 +268,20 @@ func (s *Service) handleTCPConnection(conn net.Conn) {
 	}
 }
 
+func (s *Service) trackConnection(c net.Conn) {
+	s.tcpConnectionsMu.Lock()
+	defer s.tcpConnectionsMu.Unlock()
+	s.tcpConnections[c.RemoteAddr().String()] = &tcpConnection{
+		conn:        c,
+		connectTime: time.Now().UTC(),
+	}
+}
+func (s *Service) untrackConnection(c net.Conn) {
+	s.tcpConnectionsMu.Lock()
+	defer s.tcpConnectionsMu.Unlock()
+	delete(s.tcpConnections, c.RemoteAddr().String())
+}
+
 // openUDPServer opens the Graphite input in UDP mode and starts processing incoming data.
 func (s *Service) openUDPServer() (net.Addr, error) {
 	addr, err := net.ResolveUDPAddr("udp", s.bindAddress)
@@ -298,6 +294,14 @@ func (s *Service) openUDPServer() (net.Addr, error) {
 		return nil, err
 	}
 
+	if s.udpReadBuffer != 0 {
+		err = s.udpConn.SetReadBuffer(s.udpReadBuffer)
+		if err != nil {
+			return nil, fmt.Errorf("unable to set UDP read buffer to %d: %s",
+				s.udpReadBuffer, err)
+		}
+	}
+
 	buf := make([]byte, udpBufferSize)
 	s.wg.Add(1)
 	go func() {
@@ -328,21 +332,19 @@ func (s *Service) handleLine(line string) {
 	// Parse it.
 	point, err := s.parser.Parse(line)
 	if err != nil {
-		s.logger.Printf("unable to parse line: %s", err)
+		switch err := err.(type) {
+		case *UnsupposedValueError:
+			// Graphite ignores NaN values with no error.
+			if math.IsNaN(err.Value) {
+				s.statMap.Add(statPointsNaNFail, 1)
+				return
+			}
+		}
+		s.logger.Printf("unable to parse line: %s: %s", line, err)
 		s.statMap.Add(statPointsParseFail, 1)
 		return
 	}
 
-	f, ok := point.Fields()["value"].(float64)
-	if ok {
-		// Drop NaN and +/-Inf data points since they are not supported values
-		if math.IsNaN(f) || math.IsInf(f, 0) {
-			s.logger.Printf("dropping unsupported value: '%v'", line)
-			s.statMap.Add(statPointsUnsupported, 1)
-			return
-		}
-	}
-
 	s.batcher.In() <- point
 }
 
@@ -358,7 +360,7 @@ func (s *Service) processBatches(batcher *tsdb.PointBatcher) {
 				ConsistencyLevel: s.consistencyLevel,
 				Points:           batch,
 			}); err == nil {
-				s.statMap.Add(statBatchesTrasmitted, 1)
+				s.statMap.Add(statBatchesTransmitted, 1)
 				s.statMap.Add(statPointsTransmitted, int64(len(batch)))
 			} else {
 				s.logger.Printf("failed to write point batch to database %q: %s", s.database, err)
@@ -370,3 +372,19 @@ func (s *Service) processBatches(batcher *tsdb.PointBatcher) {
 		}
 	}
 }
+
+// Diagnostics returns diagnostics of the graphite service.
+func (s *Service) Diagnostics() (*monitor.Diagnostic, error) {
+	s.tcpConnectionsMu.Lock()
+	defer s.tcpConnectionsMu.Unlock()
+
+	d := &monitor.Diagnostic{
+		Columns: []string{"local", "remote", "connect time"},
+		Rows:    make([][]interface{}, 0, len(s.tcpConnections)),
+	}
+	for _, v := range s.tcpConnections {
+		_ = v
+		d.Rows = append(d.Rows, []interface{}{v.conn.LocalAddr().String(), v.conn.RemoteAddr().String(), v.connectTime})
+	}
+	return d, nil
+}
