@@ -1,5 +1,6 @@
 package httpd
 
+// Config represents a configuration for a HTTP service.
 type Config struct {
 	Enabled          bool   `toml:"enabled"`
 	BindAddress      string `toml:"bind-address"`
@@ -7,16 +8,17 @@ type Config struct {
 	LogEnabled       bool   `toml:"log-enabled"`
 	WriteTracing     bool   `toml:"write-tracing"`
 	PprofEnabled     bool   `toml:"pprof-enabled"`
-	HttpsEnabled     bool   `toml:"https-enabled"`
-	HttpsCertificate string `toml:"https-certificate"`
+	HTTPSEnabled     bool   `toml:"https-enabled"`
+	HTTPSCertificate string `toml:"https-certificate"`
 }
 
+// NewConfig returns a new Config with default settings.
 func NewConfig() Config {
 	return Config{
 		Enabled:          true,
 		BindAddress:      ":8086",
 		LogEnabled:       true,
-		HttpsEnabled:     false,
-		HttpsCertificate: "/etc/ssl/influxdb.pem",
+		HTTPSEnabled:     false,
+		HTTPSCertificate: "/etc/ssl/influxdb.pem",
 	}
 }
