@@ -22,15 +22,18 @@ import (
 	"github.com/influxdb/influxdb/client"
 	"github.com/influxdb/influxdb/cluster"
 	"github.com/influxdb/influxdb/influxql"
-	"github.com/influxdb/influxdb/meta"
+	"github.com/influxdb/influxdb/models"
 	"github.com/influxdb/influxdb/services/continuous_querier"
-	"github.com/influxdb/influxdb/tsdb"
+	"github.com/influxdb/influxdb/services/meta"
 	"github.com/influxdb/influxdb/uuid"
 )
 
 const (
-	// With raw data queries, mappers will read up to this amount before sending results back to the engine.
-	// This is the default size in the number of values returned in a raw query. Could be many more bytes depending on fields returned.
+	// DefaultChunkSize specifies the amount of data mappers will read
+	// up to, before sending results back to the engine. This is the
+	// default size in the number of values returned in a raw query.
+	//
+	// Could be many more bytes depending on fields returned.
 	DefaultChunkSize = 10000
 )
 
@@ -54,15 +57,16 @@ type Handler struct {
 	requireAuthentication bool
 	Version               string
 
-	MetaStore interface {
+	MetaClient interface {
 		Database(name string) (*meta.DatabaseInfo, error)
 		Authenticate(username, password string) (ui *meta.UserInfo, err error)
-		Users() ([]meta.UserInfo, error)
+		Users() []meta.UserInfo
+		Ping(checkAllMetaServers bool) error
 	}
 
 	QueryExecutor interface {
 		Authorize(u *meta.UserInfo, q *influxql.Query, db string) error
-		ExecuteQuery(q *influxql.Query, db string, chunkSize int) (<-chan *influxql.Result, error)
+		ExecuteQuery(q *influxql.Query, db string, chunkSize int, closing chan struct{}) (<-chan *influxql.Result, error)
 	}
 
 	PointsWriter interface {
@@ -113,6 +117,14 @@ func NewHandler(requireAuthentication, loggingEnabled, writeTrace bool, statMap
 			"ping-head",
 			"HEAD", "/ping", true, true, h.servePing,
 		},
+		route{ // Ping w/ status
+			"status",
+			"GET", "/status", true, true, h.serveStatus,
+		},
+		route{ // Ping w/ status
+			"status-head",
+			"HEAD", "/status", true, true, h.serveStatus,
+		},
 		route{ // Tell data node to run CQs that should be run
 			"process_continuous_queries",
 			"POST", "/data/process_continuous_queries", false, false, h.serveProcessContinuousQueries,
@@ -122,6 +134,7 @@ func NewHandler(requireAuthentication, loggingEnabled, writeTrace bool, statMap
 	return h
 }
 
+// SetRoutes sets the provided routes on the handler.
 func (h *Handler) SetRoutes(routes []route) {
 	for _, r := range routes {
 		var handler http.Handler
@@ -189,7 +202,7 @@ func (h *Handler) serveProcessContinuousQueries(w http.ResponseWriter, r *http.R
 	// Get the name of the CQ to run (blank means run all).
 	name := q.Get("name")
 	// Get the time for which the CQ should be evaluated.
-	var t time.Time
+	t := time.Now()
 	var err error
 	s := q.Get("time")
 	if s != "" {
@@ -267,9 +280,19 @@ func (h *Handler) serveQuery(w http.ResponseWriter, r *http.Request, user *meta.
 		}
 	}
 
+	// Make sure if the client disconnects we signal the query to abort
+	closing := make(chan struct{})
+	if notifier, ok := w.(http.CloseNotifier); ok {
+		notify := notifier.CloseNotify()
+		go func() {
+			<-notify
+			close(closing)
+		}()
+	}
+
 	// Execute query.
 	w.Header().Add("content-type", "application/json")
-	results, err := h.QueryExecutor.ExecuteQuery(query, db, chunkSize)
+	results, err := h.QueryExecutor.ExecuteQuery(query, db, chunkSize, closing)
 
 	if err != nil {
 		w.WriteHeader(http.StatusInternalServerError)
@@ -313,17 +336,19 @@ func (h *Handler) serveQuery(w http.ResponseWriter, r *http.Request, user *meta.
 			resp.Results = append(resp.Results, r)
 		} else if resp.Results[l-1].StatementID == r.StatementID {
 			cr := resp.Results[l-1]
-			lastSeries := cr.Series[len(cr.Series)-1]
 			rowsMerged := 0
+			if len(cr.Series) > 0 {
+				lastSeries := cr.Series[len(cr.Series)-1]
 
-			for _, row := range r.Series {
-				if !lastSeries.SameSeries(row) {
-					// Next row is for a different series than last.
-					break
+				for _, row := range r.Series {
+					if !lastSeries.SameSeries(row) {
+						// Next row is for a different series than last.
+						break
+					}
+					// Values are for the same series, so append them.
+					lastSeries.Values = append(lastSeries.Values, row.Values...)
+					rowsMerged++
 				}
-				// Values are for the same series, so append them.
-				lastSeries.Values = append(lastSeries.Values, row.Values...)
-				rowsMerged++
 			}
 
 			// Append remaining rows as new rows.
@@ -349,7 +374,7 @@ func (h *Handler) serveWrite(w http.ResponseWriter, r *http.Request, user *meta.
 	if r.Header.Get("Content-encoding") == "gzip" {
 		b, err := gzip.NewReader(r.Body)
 		if err != nil {
-			h.writeError(w, influxql.Result{Err: err}, http.StatusBadRequest)
+			resultError(w, influxql.Result{Err: err}, http.StatusBadRequest)
 			return
 		}
 		body = b
@@ -361,7 +386,7 @@ func (h *Handler) serveWrite(w http.ResponseWriter, r *http.Request, user *meta.
 		if h.WriteTrace {
 			h.Logger.Print("write handler unable to read bytes from request body")
 		}
-		h.writeError(w, influxql.Result{Err: err}, http.StatusBadRequest)
+		resultError(w, influxql.Result{Err: err}, http.StatusBadRequest)
 		return
 	}
 	h.statMap.Add(statWriteRequestBytesReceived, int64(len(b)))
@@ -397,7 +422,7 @@ func (h *Handler) serveWriteJSON(w http.ResponseWriter, r *http.Request, body []
 		return
 	}
 
-	if di, err := h.MetaStore.Database(bp.Database); err != nil {
+	if di, err := h.MetaClient.Database(bp.Database); err != nil {
 		resultError(w, influxql.Result{Err: fmt.Errorf("metastore database error: %s", err)}, http.StatusInternalServerError)
 		return
 	} else if di == nil {
@@ -430,9 +455,9 @@ func (h *Handler) serveWriteJSON(w http.ResponseWriter, r *http.Request, body []
 	}); err != nil {
 		h.statMap.Add(statPointsWrittenFail, int64(len(points)))
 		if influxdb.IsClientError(err) {
-			h.writeError(w, influxql.Result{Err: err}, http.StatusBadRequest)
+			resultError(w, influxql.Result{Err: err}, http.StatusBadRequest)
 		} else {
-			h.writeError(w, influxql.Result{Err: err}, http.StatusInternalServerError)
+			resultError(w, influxql.Result{Err: err}, http.StatusInternalServerError)
 		}
 		return
 	}
@@ -441,12 +466,6 @@ func (h *Handler) serveWriteJSON(w http.ResponseWriter, r *http.Request, body []
 	w.WriteHeader(http.StatusNoContent)
 }
 
-func (h *Handler) writeError(w http.ResponseWriter, result influxql.Result, statusCode int) {
-	w.WriteHeader(statusCode)
-	w.Write([]byte(result.Err.Error()))
-	w.Write([]byte("\n"))
-}
-
 // serveWriteLine receives incoming series data in line protocol format and writes it to the database.
 func (h *Handler) serveWriteLine(w http.ResponseWriter, r *http.Request, body []byte, user *meta.UserInfo) {
 	// Some clients may not set the content-type header appropriately and send JSON with a non-json
@@ -461,10 +480,10 @@ func (h *Handler) serveWriteLine(w http.ResponseWriter, r *http.Request, body []
 			}
 
 			// check that the byte is in the standard ascii code range
-			if body[i] > 32 {
+			if body[i] > 32 || i >= len(body)-1 {
 				break
 			}
-			i += 1
+			i++
 		}
 	}
 
@@ -473,37 +492,38 @@ func (h *Handler) serveWriteLine(w http.ResponseWriter, r *http.Request, body []
 		precision = "n"
 	}
 
-	points, err := tsdb.ParsePointsWithPrecision(body, time.Now().UTC(), precision)
-	if err != nil {
-		if err.Error() == "EOF" {
+	points, parseError := models.ParsePointsWithPrecision(body, time.Now().UTC(), precision)
+	// Not points parsed correctly so return the error now
+	if parseError != nil && len(points) == 0 {
+		if parseError.Error() == "EOF" {
 			w.WriteHeader(http.StatusOK)
 			return
 		}
-		h.writeError(w, influxql.Result{Err: err}, http.StatusBadRequest)
+		resultError(w, influxql.Result{Err: parseError}, http.StatusBadRequest)
 		return
 	}
 
 	database := r.FormValue("db")
 	if database == "" {
-		h.writeError(w, influxql.Result{Err: fmt.Errorf("database is required")}, http.StatusBadRequest)
+		resultError(w, influxql.Result{Err: fmt.Errorf("database is required")}, http.StatusBadRequest)
 		return
 	}
 
-	if di, err := h.MetaStore.Database(database); err != nil {
-		h.writeError(w, influxql.Result{Err: fmt.Errorf("metastore database error: %s", err)}, http.StatusInternalServerError)
+	if di, err := h.MetaClient.Database(database); err != nil {
+		resultError(w, influxql.Result{Err: fmt.Errorf("metastore database error: %s", err)}, http.StatusInternalServerError)
 		return
 	} else if di == nil {
-		h.writeError(w, influxql.Result{Err: fmt.Errorf("database not found: %q", database)}, http.StatusNotFound)
+		resultError(w, influxql.Result{Err: fmt.Errorf("database not found: %q", database)}, http.StatusNotFound)
 		return
 	}
 
 	if h.requireAuthentication && user == nil {
-		h.writeError(w, influxql.Result{Err: fmt.Errorf("user is required to write to database %q", database)}, http.StatusUnauthorized)
+		resultError(w, influxql.Result{Err: fmt.Errorf("user is required to write to database %q", database)}, http.StatusUnauthorized)
 		return
 	}
 
 	if h.requireAuthentication && !user.Authorize(influxql.WritePrivilege, database) {
-		h.writeError(w, influxql.Result{Err: fmt.Errorf("%q user is not authorized to write to database %q", user.Name, database)}, http.StatusUnauthorized)
+		resultError(w, influxql.Result{Err: fmt.Errorf("%q user is not authorized to write to database %q", user.Name, database)}, http.StatusUnauthorized)
 		return
 	}
 
@@ -528,11 +548,18 @@ func (h *Handler) serveWriteLine(w http.ResponseWriter, r *http.Request, body []
 		Points:           points,
 	}); influxdb.IsClientError(err) {
 		h.statMap.Add(statPointsWrittenFail, int64(len(points)))
-		h.writeError(w, influxql.Result{Err: err}, http.StatusBadRequest)
+		resultError(w, influxql.Result{Err: err}, http.StatusBadRequest)
 		return
 	} else if err != nil {
 		h.statMap.Add(statPointsWrittenFail, int64(len(points)))
-		h.writeError(w, influxql.Result{Err: err}, http.StatusInternalServerError)
+		resultError(w, influxql.Result{Err: err}, http.StatusInternalServerError)
+		return
+	} else if parseError != nil {
+		// We wrote some of the points
+		h.statMap.Add(statPointsWrittenOK, int64(len(points)))
+		// The other points failed to parse which means the client sent invalid line protocol.  We return a 400
+		// response code as well as the lines that failed to parse.
+		resultError(w, influxql.Result{Err: fmt.Errorf("partial write:\n%v", parseError)}, http.StatusBadRequest)
 		return
 	}
 
@@ -551,6 +578,18 @@ func (h *Handler) servePing(w http.ResponseWriter, r *http.Request) {
 	w.WriteHeader(http.StatusNoContent)
 }
 
+// serveStatus returns a simple response to let the client know the whole cluster is running.
+func (h *Handler) serveStatus(w http.ResponseWriter, r *http.Request) {
+	h.statMap.Add(statStatusRequest, 1)
+
+	if err := h.MetaClient.Ping(false); err != nil {
+		w.WriteHeader(http.StatusServiceUnavailable)
+		return
+	}
+
+	w.WriteHeader(http.StatusNoContent)
+}
+
 // convertToEpoch converts result timestamps from time.Time to the specified epoch.
 func convertToEpoch(r *influxql.Result, epoch string) {
 	divisor := int64(1)
@@ -593,6 +632,7 @@ func MarshalJSON(v interface{}, pretty bool) []byte {
 	return b
 }
 
+// Point represents an InfluxDB point.
 type Point struct {
 	Name   string                 `json:"name"`
 	Time   time.Time              `json:"time"`
@@ -600,6 +640,8 @@ type Point struct {
 	Fields map[string]interface{} `json:"fields"`
 }
 
+// Batch is a collection of points associated with a database, having a
+// certain retention policy.
 type Batch struct {
 	Database        string  `json:"database"`
 	RetentionPolicy string  `json:"retentionPolicy"`
@@ -676,11 +718,7 @@ func authenticate(inner func(http.ResponseWriter, *http.Request, *meta.UserInfo)
 		var user *meta.UserInfo
 
 		// Retrieve user list.
-		uis, err := h.MetaStore.Users()
-		if err != nil {
-			httpError(w, err.Error(), false, http.StatusInternalServerError)
-			return
-		}
+		uis := h.MetaClient.Users()
 
 		// TODO corylanou: never allow this in the future without users
 		if requireAuthentication && len(uis) > 0 {
@@ -696,7 +734,7 @@ func authenticate(inner func(http.ResponseWriter, *http.Request, *meta.UserInfo)
 				return
 			}
 
-			user, err = h.MetaStore.Authenticate(username, password)
+			user, err = h.MetaClient.Authenticate(username, password)
 			if err != nil {
 				h.statMap.Add(statAuthFail, 1)
 				httpError(w, err.Error(), false, http.StatusUnauthorized)
@@ -767,6 +805,11 @@ func cors(inner http.Handler) http.Handler {
 				`X-CSRF-Token`,
 				`X-HTTP-Method-Override`,
 			}, ", "))
+
+			w.Header().Set(`Access-Control-Expose-Headers`, strings.Join([]string{
+				`Date`,
+				`X-Influxdb-Version`,
+			}, ", "))
 		}
 
 		if r.Method == "OPTIONS" {
@@ -801,12 +844,16 @@ func recovery(inner http.Handler, name string, weblog *log.Logger) http.Handler
 	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		start := time.Now()
 		l := &responseLogger{w: w}
+
+		defer func() {
+			if err := recover(); err != nil {
+				logLine := buildLogLine(l, r, start)
+				logLine = fmt.Sprintf(`%s [panic:%s]`, logLine, err)
+				weblog.Println(logLine)
+			}
+		}()
+
 		inner.ServeHTTP(l, r)
-		if err := recover(); err != nil {
-			logLine := buildLogLine(l, r, start)
-			logLine = fmt.Sprintf(`%s [err:%s]`, logLine, err)
-			weblog.Println(logLine)
-		}
 	})
 }
 
@@ -868,8 +915,8 @@ func (r *Response) Error() error {
 // NormalizeBatchPoints returns a slice of Points, created by populating individual
 // points within the batch, which do not have times or tags, with the top-level
 // values.
-func NormalizeBatchPoints(bp client.BatchPoints) ([]tsdb.Point, error) {
-	points := []tsdb.Point{}
+func NormalizeBatchPoints(bp client.BatchPoints) ([]models.Point, error) {
+	points := []models.Point{}
 	for _, p := range bp.Points {
 		if p.Time.IsZero() {
 			if bp.Time.IsZero() {
@@ -901,7 +948,11 @@ func NormalizeBatchPoints(bp client.BatchPoints) ([]tsdb.Point, error) {
 			return points, fmt.Errorf("missing fields")
 		}
 		// Need to convert from a client.Point to a influxdb.Point
-		points = append(points, tsdb.NewPoint(p.Measurement, p.Tags, p.Fields, p.Time))
+		pt, err := models.NewPoint(p.Measurement, p.Tags, p.Fields, p.Time)
+		if err != nil {
+			return points, err
+		}
+		points = append(points, pt)
 	}
 
 	return points, nil
