@@ -15,9 +15,11 @@ func TestConfig_Parse(t *testing.T) {
 enabled = true
 bind-address = ":4444"
 database = "awesomedb"
+retention-policy = "awesomerp"
 batch-size = 100
 batch-pending = 9
 batch-timeout = "10ms"
+udp-payload-size = 1500
 `, &c); err != nil {
 		t.Fatal(err)
 	}
@@ -29,11 +31,15 @@ batch-timeout = "10ms"
 		t.Fatalf("unexpected bind address: %s", c.BindAddress)
 	} else if c.Database != "awesomedb" {
 		t.Fatalf("unexpected database: %s", c.Database)
+	} else if c.RetentionPolicy != "awesomerp" {
+		t.Fatalf("unexpected retention policy: %s", c.RetentionPolicy)
 	} else if c.BatchSize != 100 {
 		t.Fatalf("unexpected batch size: %d", c.BatchSize)
 	} else if c.BatchPending != 9 {
 		t.Fatalf("unexpected batch pending: %d", c.BatchPending)
 	} else if time.Duration(c.BatchTimeout) != (10 * time.Millisecond) {
 		t.Fatalf("unexpected batch timeout: %v", c.BatchTimeout)
+	} else if c.UDPPayloadSize != 1500 {
+		t.Fatalf("unexpected udp-payload-size: %d", c.UDPPayloadSize)
 	}
 }
