@@ -9,6 +9,7 @@ import (
 	"time"
 
 	"github.com/influxdb/influxdb/influxql"
+	"github.com/influxdb/influxdb/pkg/escape"
 	"github.com/influxdb/influxdb/tsdb/internal"
 
 	"github.com/gogo/protobuf/proto"
@@ -58,6 +59,20 @@ func (d *DatabaseIndex) Measurement(name string) *Measurement {
 	return d.measurements[name]
 }
 
+// MeasurementsByName returns a list of measurements.
+func (d *DatabaseIndex) MeasurementsByName(names []string) []*Measurement {
+	d.mu.RLock()
+	defer d.mu.RUnlock()
+
+	a := make([]*Measurement, 0, len(names))
+	for _, name := range names {
+		if m := d.measurements[name]; m != nil {
+			a = append(a, m)
+		}
+	}
+	return a
+}
+
 // MeasurementSeriesCounts returns the number of measurements and series currently indexed by the database.
 // Useful for reporting and monitoring.
 func (d *DatabaseIndex) MeasurementSeriesCounts() (nMeasurements int, nSeries int) {
@@ -92,7 +107,7 @@ func (s *DatabaseIndex) CreateSeriesIndexIfNotExists(measurementName string, ser
 
 // CreateMeasurementIndexIfNotExists creates or retrieves an in memory index object for the measurement
 func (s *DatabaseIndex) CreateMeasurementIndexIfNotExists(name string) *Measurement {
-	name = unescapeString(name)
+	name = escape.UnescapeString(name)
 	m := s.measurements[name]
 	if m == nil {
 		m = NewMeasurement(name, s)
@@ -121,7 +136,7 @@ func (db *DatabaseIndex) measurementsByExpr(expr influxql.Expr) (Measurements, e
 		case influxql.EQ, influxql.NEQ, influxql.EQREGEX, influxql.NEQREGEX:
 			tag, ok := e.LHS.(*influxql.VarRef)
 			if !ok {
-				return nil, fmt.Errorf("left side of '%s' must be a tag name", e.Op.String())
+				return nil, fmt.Errorf("left side of '%s' must be a tag key", e.Op.String())
 			}
 
 			tf := &TagFilter{
@@ -169,7 +184,7 @@ func (db *DatabaseIndex) measurementsByExpr(expr influxql.Expr) (Measurements, e
 	return nil, fmt.Errorf("%#v", expr)
 }
 
-// measurementsByTagFilters returns the measurements matching the filters on tag values.
+// measurementsByTagFilters returns the sorted measurements matching the filters on tag values.
 func (db *DatabaseIndex) measurementsByTagFilters(filters []*TagFilter) Measurements {
 	// If no filters, then return all measurements.
 	if len(filters) == 0 {
@@ -227,6 +242,7 @@ func (db *DatabaseIndex) measurementsByTagFilters(filters []*TagFilter) Measurem
 		}
 	}
 
+	sort.Sort(measurements)
 	return measurements
 }
 
@@ -280,6 +296,142 @@ func (db *DatabaseIndex) DropSeries(keys []string) {
 	}
 }
 
+// RewriteSelectStatement performs any necessary query re-writing.
+func (db *DatabaseIndex) RewriteSelectStatement(stmt *influxql.SelectStatement) (*influxql.SelectStatement, error) {
+	// Expand regex expressions in the FROM clause.
+	sources, err := db.ExpandSources(stmt.Sources)
+	if err != nil {
+		return nil, err
+	}
+	stmt.Sources = sources
+
+	// Expand wildcards in the fields or GROUP BY.
+	stmt, err = db.ExpandWildcards(stmt)
+	if err != nil {
+		return nil, err
+	}
+
+	stmt.RewriteDistinct()
+
+	return stmt, nil
+}
+
+// expandWildcards returns a new SelectStatement with wildcards expanded
+// If only a `SELECT *` is present, without a `GROUP BY *`, both tags and fields expand in the SELECT
+// If a `SELECT *` and a `GROUP BY *` are both present, then only fiels are expanded in the `SELECT` and only
+// tags are expanded in the `GROUP BY`
+func (db *DatabaseIndex) ExpandWildcards(stmt *influxql.SelectStatement) (*influxql.SelectStatement, error) {
+	// If there are no wildcards in the statement, return it as-is.
+	if !stmt.HasWildcard() {
+		return stmt, nil
+	}
+	// Use sets to avoid duplicate field names.
+	fieldSet := map[string]struct{}{}
+	dimensionSet := map[string]struct{}{}
+
+	// keep track of where the wildcards are in the select statement
+	hasFieldWildcard := stmt.HasFieldWildcard()
+	hasDimensionWildcard := stmt.HasDimensionWildcard()
+
+	// Iterate measurements in the FROM clause getting the fields & dimensions for each.
+	var fields influxql.Fields
+	var dimensions influxql.Dimensions
+	for _, src := range stmt.Sources {
+		if m, ok := src.(*influxql.Measurement); ok {
+			// Lookup the measurement in the database.
+			mm := db.Measurement(m.Name)
+			if mm == nil {
+				// This shard have never received data for the measurement. No Mapper
+				// required.
+				return stmt, nil
+			}
+
+			// Get the fields for this measurement.
+			for _, name := range mm.FieldNames() {
+				if _, ok := fieldSet[name]; ok {
+					continue
+				}
+				fieldSet[name] = struct{}{}
+				fields = append(fields, &influxql.Field{Expr: &influxql.VarRef{Val: name}})
+			}
+
+			// Add tags to fields if a field wildcard was provided and a dimension wildcard was not.
+			if hasFieldWildcard && !hasDimensionWildcard {
+				for _, t := range mm.TagKeys() {
+					if _, ok := fieldSet[t]; ok {
+						continue
+					}
+					fieldSet[t] = struct{}{}
+					fields = append(fields, &influxql.Field{Expr: &influxql.VarRef{Val: t}})
+				}
+			}
+
+			// Get the dimensions for this measurement.
+			if hasDimensionWildcard {
+				for _, t := range mm.TagKeys() {
+					if _, ok := dimensionSet[t]; ok {
+						continue
+					}
+					dimensionSet[t] = struct{}{}
+					dimensions = append(dimensions, &influxql.Dimension{Expr: &influxql.VarRef{Val: t}})
+				}
+			}
+		}
+	}
+
+	// Return a new SelectStatement with the wild cards rewritten.
+	return stmt.RewriteWildcards(fields, dimensions), nil
+}
+
+// expandSources expands regex sources and removes duplicates.
+// NOTE: sources must be normalized (db and rp set) before calling this function.
+func (di *DatabaseIndex) ExpandSources(sources influxql.Sources) (influxql.Sources, error) {
+	// Use a map as a set to prevent duplicates. Two regexes might produce
+	// duplicates when expanded.
+	set := map[string]influxql.Source{}
+	names := []string{}
+
+	// Iterate all sources, expanding regexes when they're found.
+	for _, source := range sources {
+		switch src := source.(type) {
+		case *influxql.Measurement:
+			if src.Regex == nil {
+				name := src.String()
+				set[name] = src
+				names = append(names, name)
+				continue
+			}
+			// Get measurements from the database that match the regex.
+			measurements := di.measurementsByRegex(src.Regex.Val)
+			// Add those measurements to the set.
+			for _, m := range measurements {
+				m2 := &influxql.Measurement{
+					Database:        src.Database,
+					RetentionPolicy: src.RetentionPolicy,
+					Name:            m.Name,
+				}
+				name := m2.String()
+				if _, ok := set[name]; !ok {
+					set[name] = m2
+					names = append(names, name)
+				}
+			}
+		default:
+			return nil, fmt.Errorf("expandSources: unsuported source type: %T", source)
+		}
+	}
+
+	// Sort the list of source names.
+	sort.Strings(names)
+
+	// Convert set to a list of Sources.
+	expanded := make(influxql.Sources, 0, len(set))
+	for _, name := range names {
+		expanded = append(expanded, set[name])
+	}
+	return expanded, nil
+}
+
 // Measurement represents a collection of time series in a database. It also contains in memory
 // structures for indexing tags. Exported functions are goroutine safe while un-exported functions
 // assume the caller will use the appropriate locks
@@ -451,7 +603,7 @@ func (m *Measurement) DropSeries(seriesID uint64) {
 // filters walks the where clause of a select statement and returns a map with all series ids
 // matching the where clause and any filter expression that should be applied to each
 func (m *Measurement) filters(stmt *influxql.SelectStatement) (map[uint64]influxql.Expr, error) {
-	if stmt.Condition == nil || stmt.OnlyTimeDimensions() {
+	if stmt.Condition == nil || influxql.OnlyTimeExpr(stmt.Condition) {
 		seriesIdsToExpr := make(map[uint64]influxql.Expr)
 		for _, id := range m.seriesIDs {
 			seriesIdsToExpr[id] = nil
@@ -547,7 +699,7 @@ func (m *Measurement) TagSets(stmt *influxql.SelectStatement, dimensions []strin
 }
 
 // mergeSeriesFilters merges two sets of filter expressions and culls series IDs.
-func mergeSeriesFilters(op influxql.Token, ids SeriesIDs, lfilters, rfilters map[uint64]influxql.Expr) (SeriesIDs, map[uint64]influxql.Expr) {
+func mergeSeriesFilters(op influxql.Token, ids SeriesIDs, lfilters, rfilters FilterExprs) (SeriesIDs, FilterExprs) {
 	// Create a map to hold the final set of series filter expressions.
 	filters := make(map[uint64]influxql.Expr, 0)
 	// Resulting list of series IDs
@@ -681,10 +833,30 @@ func (m *Measurement) idsForExpr(n *influxql.BinaryExpr) (SeriesIDs, influxql.Ex
 	return nil, nil, nil
 }
 
+// FilterExprs represents a map of series IDs to filter expressions.
+type FilterExprs map[uint64]influxql.Expr
+
+// DeleteBoolLiteralTrues deletes all elements whose filter expression is a boolean literal true.
+func (fe FilterExprs) DeleteBoolLiteralTrues() {
+	for id, expr := range fe {
+		if e, ok := expr.(*influxql.BooleanLiteral); ok && e.Val == true {
+			delete(fe, id)
+		}
+	}
+}
+
+// Len returns the number of elements.
+func (fe FilterExprs) Len() int {
+	if fe == nil {
+		return 0
+	}
+	return len(fe)
+}
+
 // walkWhereForSeriesIds recursively walks the WHERE clause and returns an ordered set of series IDs and
 // a map from those series IDs to filter expressions that should be used to limit points returned in
 // the final query result.
-func (m *Measurement) walkWhereForSeriesIds(expr influxql.Expr) (SeriesIDs, map[uint64]influxql.Expr, error) {
+func (m *Measurement) walkWhereForSeriesIds(expr influxql.Expr) (SeriesIDs, FilterExprs, error) {
 	switch n := expr.(type) {
 	case *influxql.BinaryExpr:
 		switch n.Op {
@@ -695,7 +867,7 @@ func (m *Measurement) walkWhereForSeriesIds(expr influxql.Expr) (SeriesIDs, map[
 				return nil, nil, err
 			}
 
-			filters := map[uint64]influxql.Expr{}
+			filters := FilterExprs{}
 			for _, id := range ids {
 				filters[id] = expr
 			}
@@ -901,6 +1073,64 @@ func (m *Measurement) uniqueTagValues(expr influxql.Expr) map[string][]string {
 	return out
 }
 
+// SelectFields returns a list of fields in the SELECT section of stmt.
+func (m *Measurement) SelectFields(stmt *influxql.SelectStatement) []string {
+	set := newStringSet()
+	for _, name := range stmt.NamesInSelect() {
+		if m.HasField(name) {
+			set.add(name)
+			continue
+		}
+	}
+	return set.list()
+}
+
+// SelectTags returns a list of non-field tags in the SELECT section of stmt.
+func (m *Measurement) SelectTags(stmt *influxql.SelectStatement) []string {
+	set := newStringSet()
+	for _, name := range stmt.NamesInSelect() {
+		if !m.HasField(name) && m.HasTagKey(name) {
+			set.add(name)
+		}
+	}
+	return set.list()
+}
+
+// WhereFields returns a list of non-"time" fields in the WHERE section of stmt.
+func (m *Measurement) WhereFields(stmt *influxql.SelectStatement) []string {
+	set := newStringSet()
+	for _, name := range stmt.NamesInWhere() {
+		if name != "time" && m.HasField(name) {
+			set.add(name)
+		}
+	}
+	return set.list()
+}
+
+// DimensionTagSets returns list of tag sets from the GROUP BY section of stmt.
+func (m *Measurement) DimensionTagSets(stmt *influxql.SelectStatement) ([]*influxql.TagSet, error) {
+	_, tagKeys := stmt.Dimensions.Normalize()
+
+	for _, n := range stmt.NamesInDimension() {
+		if m.HasTagKey(n) {
+			tagKeys = append(tagKeys, n)
+		}
+	}
+
+	// Get the sorted unique tag sets for this statement.
+	tagSets, err := m.TagSets(stmt, tagKeys)
+	if err != nil {
+		return nil, err
+	}
+	return tagSets, nil
+}
+
+type SelectInfo struct {
+	SelectFields []string
+	SelectTags   []string
+	WhereFields  []string
+}
+
 // Measurements represents a list of *Measurement.
 type Measurements []*Measurement
 
@@ -908,6 +1138,45 @@ func (a Measurements) Len() int           { return len(a) }
 func (a Measurements) Less(i, j int) bool { return a[i].Name < a[j].Name }
 func (a Measurements) Swap(i, j int)      { a[i], a[j] = a[j], a[i] }
 
+// SelectFields returns a list of fields in the SELECT section of stmt.
+func (a Measurements) SelectFields(stmt *influxql.SelectStatement) []string {
+	set := newStringSet()
+	for _, name := range stmt.NamesInSelect() {
+		for _, m := range a {
+			if m.HasField(name) {
+				set.add(name)
+			}
+		}
+	}
+	return set.list()
+}
+
+// SelectTags returns a list of non-field tags in the SELECT section of stmt.
+func (a Measurements) SelectTags(stmt *influxql.SelectStatement) []string {
+	set := newStringSet()
+	for _, name := range stmt.NamesInSelect() {
+		for _, m := range a {
+			if !m.HasField(name) && m.HasTagKey(name) {
+				set.add(name)
+			}
+		}
+	}
+	return set.list()
+}
+
+// WhereFields returns a list of non-"time" fields in the WHERE section of stmt.
+func (a Measurements) WhereFields(stmt *influxql.SelectStatement) []string {
+	set := newStringSet()
+	for _, name := range stmt.NamesInWhere() {
+		for _, m := range a {
+			if name != "time" && m.HasField(name) {
+				set.add(name)
+			}
+		}
+	}
+	return set.list()
+}
+
 func (a Measurements) intersect(other Measurements) Measurements {
 	l := a
 	r := other
