@@ -6,11 +6,11 @@ import (
 	"log"
 	"os"
 	"sort"
-	"strings"
 	"time"
 
 	"github.com/influxdb/influxdb/influxql"
-	"github.com/influxdb/influxdb/meta"
+	"github.com/influxdb/influxdb/models"
+	"github.com/influxdb/influxdb/services/meta"
 )
 
 // QueryExecutor executes every statement in an influxdb Query. It is responsible for
@@ -19,20 +19,15 @@ import (
 // in a running process
 type QueryExecutor struct {
 	// The meta store for accessing and updating cluster and schema data.
-	MetaStore interface {
+	MetaClient interface {
 		Database(name string) (*meta.DatabaseInfo, error)
 		Databases() ([]meta.DatabaseInfo, error)
 		User(name string) (*meta.UserInfo, error)
-		AdminUserExists() (bool, error)
+		AdminUserExists() bool
 		Authenticate(username, password string) (*meta.UserInfo, error)
 		RetentionPolicy(database, name string) (rpi *meta.RetentionPolicyInfo, err error)
-		UserCount() (int, error)
+		UserCount() int
 		ShardGroupsByTimeRange(database, policy string, min, max time.Time) (a []meta.ShardGroupInfo, err error)
-		NodeID() uint64
-	}
-
-	// Executes statements relating to meta data.
-	MetaStatementExecutor interface {
 		ExecuteStatement(stmt influxql.Statement) *influxql.Result
 	}
 
@@ -46,12 +41,24 @@ type QueryExecutor struct {
 		CreateMapper(shard meta.ShardInfo, stmt influxql.Statement, chunkSize int) (Mapper, error)
 	}
 
-	Logger *log.Logger
+	IntoWriter interface {
+		WritePointsInto(p *IntoWriteRequest) error
+	}
+
+	Logger          *log.Logger
+	QueryLogEnabled bool
 
 	// the local data store
 	Store *Store
 }
 
+// partial copy of cluster.WriteRequest
+type IntoWriteRequest struct {
+	Database        string
+	RetentionPolicy string
+	Points          []models.Point
+}
+
 // NewQueryExecutor returns an initialized QueryExecutor
 func NewQueryExecutor(store *Store) *QueryExecutor {
 	return &QueryExecutor{
@@ -71,7 +78,7 @@ func (q *QueryExecutor) SetLogger(l *log.Logger) {
 // a root user.
 func (q *QueryExecutor) Authorize(u *meta.UserInfo, query *influxql.Query, database string) error {
 	// Special case if no users exist.
-	if count, err := q.MetaStore.UserCount(); count == 0 && err == nil {
+	if count := q.MetaClient.UserCount(); count == 0 {
 		// Ensure there is at least one statement.
 		if len(query.Statements) > 0 {
 			// First statement in the query must create a user with admin privilege.
@@ -126,7 +133,7 @@ func (q *QueryExecutor) Authorize(u *meta.UserInfo, query *influxql.Query, datab
 // ExecuteQuery executes an InfluxQL query against the server.
 // It sends results down the passed in chan and closes it when done. It will close the chan
 // on the first statement that throws an error.
-func (q *QueryExecutor) ExecuteQuery(query *influxql.Query, database string, chunkSize int) (<-chan *influxql.Result, error) {
+func (q *QueryExecutor) ExecuteQuery(query *influxql.Query, database string, chunkSize int, closing chan struct{}) (<-chan *influxql.Result, error) {
 	// Execute each statement. Keep the iterator external so we can
 	// track how many of the statements were executed
 	results := make(chan *influxql.Result)
@@ -151,12 +158,14 @@ func (q *QueryExecutor) ExecuteQuery(query *influxql.Query, database string, chu
 			}
 
 			// Log each normalized statement.
-			q.Logger.Println(stmt.String())
+			if q.QueryLogEnabled {
+				q.Logger.Println(stmt.String())
+			}
 
 			var res *influxql.Result
 			switch stmt := stmt.(type) {
 			case *influxql.SelectStatement:
-				if err := q.executeSelectStatement(i, stmt, results, chunkSize); err != nil {
+				if err := q.executeStatement(i, stmt, database, results, chunkSize, closing); err != nil {
 					results <- &influxql.Result{Err: err}
 					break
 				}
@@ -169,12 +178,15 @@ func (q *QueryExecutor) ExecuteQuery(query *influxql.Query, database string, chu
 				// TODO: handle this in a cluster
 				res = q.executeDropMeasurementStatement(stmt, database)
 			case *influxql.ShowMeasurementsStatement:
-				if err := q.executeShowMeasurementsStatement(i, stmt, database, results, chunkSize); err != nil {
+				if err := q.executeStatement(i, stmt, database, results, chunkSize, closing); err != nil {
 					results <- &influxql.Result{Err: err}
 					break
 				}
 			case *influxql.ShowTagKeysStatement:
-				res = q.executeShowTagKeysStatement(stmt, database)
+				if err := q.executeStatement(i, stmt, database, results, chunkSize, closing); err != nil {
+					results <- &influxql.Result{Err: err}
+					break
+				}
 			case *influxql.ShowTagValuesStatement:
 				res = q.executeShowTagValuesStatement(stmt, database)
 			case *influxql.ShowFieldKeysStatement:
@@ -189,7 +201,7 @@ func (q *QueryExecutor) ExecuteQuery(query *influxql.Query, database string, chu
 				res = q.MonitorStatementExecutor.ExecuteStatement(stmt)
 			default:
 				// Delegate all other meta statements to a separate executor. They don't hit tsdb storage.
-				res = q.MetaStatementExecutor.ExecuteStatement(stmt)
+				res = q.MetaClient.ExecuteStatement(stmt)
 			}
 
 			if res != nil {
@@ -217,6 +229,7 @@ func (q *QueryExecutor) ExecuteQuery(query *influxql.Query, database string, chu
 
 // Plan creates an execution plan for the given SelectStatement and returns an Executor.
 func (q *QueryExecutor) PlanSelect(stmt *influxql.SelectStatement, chunkSize int) (Executor, error) {
+	var shardIDs []uint64
 	shards := map[uint64]meta.ShardInfo{} // Shards requiring mappers.
 
 	// It is important to "stamp" this time so that everywhere we evaluate `now()` in the statement is EXACTLY the same `now`
@@ -240,20 +253,28 @@ func (q *QueryExecutor) PlanSelect(stmt *influxql.SelectStatement, chunkSize int
 
 		// Build the set of target shards. Using shard IDs as keys ensures each shard ID
 		// occurs only once.
-		shardGroups, err := q.MetaStore.ShardGroupsByTimeRange(mm.Database, mm.RetentionPolicy, tmin, tmax)
+		shardGroups, err := q.MetaClient.ShardGroupsByTimeRange(mm.Database, mm.RetentionPolicy, tmin, tmax)
 		if err != nil {
 			return nil, err
 		}
 		for _, g := range shardGroups {
 			for _, sh := range g.Shards {
-				shards[sh.ID] = sh
+				if _, ok := shards[sh.ID]; !ok {
+					shards[sh.ID] = sh
+					shardIDs = append(shardIDs, sh.ID)
+				}
 			}
 		}
 	}
 
+	// Sort shard IDs to make testing deterministic.
+	sort.Sort(uint64Slice(shardIDs))
+
 	// Build the Mappers, one per shard.
 	mappers := []Mapper{}
-	for _, sh := range shards {
+	for _, shardID := range shardIDs {
+		sh := shards[shardID]
+
 		m, err := q.ShardMapper.CreateMapper(sh, stmt, chunkSize)
 		if err != nil {
 			return nil, err
@@ -265,36 +286,16 @@ func (q *QueryExecutor) PlanSelect(stmt *influxql.SelectStatement, chunkSize int
 		mappers = append(mappers, m)
 	}
 
-	executor := NewSelectExecutor(stmt, mappers, chunkSize)
-	return executor, nil
-}
+	// Certain operations on the SELECT statement can be performed by the AggregateExecutor without
+	// assistance from the Mappers. This allows the AggregateExecutor to prepare aggregation functions
+	// and mathematical functions.
+	stmt.RewriteDistinct()
 
-// executeSelectStatement plans and executes a select statement against a database.
-func (q *QueryExecutor) executeSelectStatement(statementID int, stmt *influxql.SelectStatement, results chan *influxql.Result, chunkSize int) error {
-	// Plan statement execution.
-	e, err := q.PlanSelect(stmt, chunkSize)
-	if err != nil {
-		return err
+	if (stmt.IsRawQuery && !stmt.HasDistinct()) || stmt.IsSimpleDerivative() {
+		return NewRawExecutor(stmt, mappers, chunkSize), nil
+	} else {
+		return NewAggregateExecutor(stmt, mappers), nil
 	}
-
-	// Execute plan.
-	ch := e.Execute()
-
-	// Stream results from the channel. We should send an empty result if nothing comes through.
-	resultSent := false
-	for row := range ch {
-		if row.Err != nil {
-			return row.Err
-		}
-		resultSent = true
-		results <- &influxql.Result{StatementID: statementID, Series: []*influxql.Row{row}}
-	}
-
-	if !resultSent {
-		results <- &influxql.Result{StatementID: statementID, Series: make([]*influxql.Row, 0)}
-	}
-
-	return nil
 }
 
 // expandSources expands regex sources and removes duplicates.
@@ -360,10 +361,13 @@ func (q *QueryExecutor) expandSources(sources influxql.Sources) (influxql.Source
 // executeDropDatabaseStatement closes all local shards for the database and removes the directory. It then calls to the metastore to remove the database from there.
 // TODO: make this work in a cluster/distributed
 func (q *QueryExecutor) executeDropDatabaseStatement(stmt *influxql.DropDatabaseStatement) *influxql.Result {
-	dbi, err := q.MetaStore.Database(stmt.Name)
+	dbi, err := q.MetaClient.Database(stmt.Name)
 	if err != nil {
 		return &influxql.Result{Err: err}
 	} else if dbi == nil {
+		if stmt.IfExists {
+			return &influxql.Result{}
+		}
 		return &influxql.Result{Err: ErrDatabaseNotFound(stmt.Name)}
 	}
 
@@ -376,12 +380,17 @@ func (q *QueryExecutor) executeDropDatabaseStatement(stmt *influxql.DropDatabase
 		}
 	}
 
+	// Remove database from meta-store first so that in-flight writes can complete without error, but new ones will
+	// be rejected.
+	res := q.MetaClient.ExecuteStatement(stmt)
+
+	// Remove the database from the local store
 	err = q.Store.DeleteDatabase(stmt.Name, shardIDs)
 	if err != nil {
 		return &influxql.Result{Err: err}
 	}
 
-	return q.MetaStatementExecutor.ExecuteStatement(stmt)
+	return res
 }
 
 // executeDropMeasurementStatement removes the measurement and all series data from the local store for the given measurement
@@ -401,7 +410,7 @@ func (q *QueryExecutor) executeDropMeasurementStatement(stmt *influxql.DropMeasu
 	db.DropMeasurement(m.Name)
 
 	// now drop the raw data
-	if err := q.Store.deleteMeasurement(m.Name, m.SeriesKeys()); err != nil {
+	if err := q.Store.deleteMeasurement(database, m.Name, m.SeriesKeys()); err != nil {
 		return &influxql.Result{Err: err}
 	}
 
@@ -410,6 +419,11 @@ func (q *QueryExecutor) executeDropMeasurementStatement(stmt *influxql.DropMeasu
 
 // executeDropSeriesStatement removes all series from the local store that match the drop query
 func (q *QueryExecutor) executeDropSeriesStatement(stmt *influxql.DropSeriesStatement, database string) *influxql.Result {
+	// Check for time in WHERE clause (not supported).
+	if influxql.HasTimeExpr(stmt.Condition) {
+		return &influxql.Result{Err: errors.New("DROP SERIES doesn't support time in WHERE clause")}
+	}
+
 	// Find the database.
 	db := q.Store.DatabaseIndex(database)
 	if db == nil {
@@ -420,6 +434,8 @@ func (q *QueryExecutor) executeDropSeriesStatement(stmt *influxql.DropSeriesStat
 	sources, err := q.expandSources(stmt.Sources)
 	if err != nil {
 		return &influxql.Result{Err: err}
+	} else if stmt.Sources != nil && len(stmt.Sources) != 0 && len(sources) == 0 {
+		return &influxql.Result{}
 	}
 
 	measurements, err := measurementsFromSourcesOrDB(db, sources...)
@@ -430,12 +446,23 @@ func (q *QueryExecutor) executeDropSeriesStatement(stmt *influxql.DropSeriesStat
 	var seriesKeys []string
 	for _, m := range measurements {
 		var ids SeriesIDs
+		var filters FilterExprs
 		if stmt.Condition != nil {
 			// Get series IDs that match the WHERE clause.
-			ids, _, err = m.walkWhereForSeriesIds(stmt.Condition)
+			ids, filters, err = m.walkWhereForSeriesIds(stmt.Condition)
 			if err != nil {
 				return &influxql.Result{Err: err}
 			}
+
+			// Delete boolean literal true filter expressions.
+			// These are returned for `WHERE tagKey = 'tagVal'` type expressions and are okay.
+			filters.DeleteBoolLiteralTrues()
+
+			// Check for unsupported field filters.
+			// Any remaining filters means there were fields (e.g., `WHERE value = 1.2`).
+			if filters.Len() > 0 {
+				return &influxql.Result{Err: errors.New("DROP SERIES doesn't support fields in WHERE clause")}
+			}
 		} else {
 			// No WHERE clause so get all series IDs for this measurement.
 			ids = m.seriesIDs
@@ -447,7 +474,7 @@ func (q *QueryExecutor) executeDropSeriesStatement(stmt *influxql.DropSeriesStat
 	}
 
 	// delete the raw series data
-	if err := q.Store.deleteSeries(seriesKeys); err != nil {
+	if err := q.Store.deleteSeries(database, seriesKeys); err != nil {
 		return &influxql.Result{Err: err}
 	}
 	// remove them from the index
@@ -457,6 +484,11 @@ func (q *QueryExecutor) executeDropSeriesStatement(stmt *influxql.DropSeriesStat
 }
 
 func (q *QueryExecutor) executeShowSeriesStatement(stmt *influxql.ShowSeriesStatement, database string) *influxql.Result {
+	// Check for time in WHERE clause (not supported).
+	if influxql.HasTimeExpr(stmt.Condition) {
+		return &influxql.Result{Err: errors.New("SHOW SERIES doesn't support time in WHERE clause")}
+	}
+
 	// Find the database.
 	db := q.Store.DatabaseIndex(database)
 	if db == nil {
@@ -477,33 +509,40 @@ func (q *QueryExecutor) executeShowSeriesStatement(stmt *influxql.ShowSeriesStat
 
 	// Create result struct that will be populated and returned.
 	result := &influxql.Result{
-		Series: make(influxql.Rows, 0, len(measurements)),
+		Series: make(models.Rows, 0, len(measurements)),
 	}
 
 	// Loop through measurements to build result. One result row / measurement.
 	for _, m := range measurements {
 		var ids SeriesIDs
+		var filters FilterExprs
 
 		if stmt.Condition != nil {
 			// Get series IDs that match the WHERE clause.
-			ids, _, err = m.walkWhereForSeriesIds(stmt.Condition)
+			ids, filters, err = m.walkWhereForSeriesIds(stmt.Condition)
 			if err != nil {
 				return &influxql.Result{Err: err}
 			}
 
+			// Delete boolean literal true filter expressions.
+			filters.DeleteBoolLiteralTrues()
+
+			// Check for unsupported field filters.
+			if filters.Len() > 0 {
+				return &influxql.Result{Err: errors.New("SHOW SERIES doesn't support fields in WHERE clause")}
+			}
+
 			// If no series matched, then go to the next measurement.
 			if len(ids) == 0 {
 				continue
 			}
-
-			// TODO: check return of walkWhereForSeriesIds for fields
 		} else {
 			// No WHERE clause so get all series IDs for this measurement.
 			ids = m.seriesIDs
 		}
 
 		// Make a new row for this measurement.
-		r := &influxql.Row{
+		r := &models.Row{
 			Name:    m.Name,
 			Columns: m.TagKeys(),
 		}
@@ -541,8 +580,8 @@ func (q *QueryExecutor) executeShowSeriesStatement(stmt *influxql.ShowSeriesStat
 // filterShowSeriesResult will limit the number of series returned based on the limit and the offset.
 // Unlike limit and offset on SELECT statements, the limit and offset don't apply to the number of Rows, but
 // to the number of total Values returned, since each Value represents a unique series.
-func (q *QueryExecutor) filterShowSeriesResult(limit, offset int, rows influxql.Rows) influxql.Rows {
-	var filteredSeries influxql.Rows
+func (q *QueryExecutor) filterShowSeriesResult(limit, offset int, rows models.Rows) models.Rows {
+	var filteredSeries models.Rows
 	seriesCount := 0
 	for _, r := range rows {
 		var currentSeries [][]interface{}
@@ -567,10 +606,28 @@ func (q *QueryExecutor) filterShowSeriesResult(limit, offset int, rows influxql.
 	return filteredSeries
 }
 
-// PlanShowMeasurements creates an execution plan for the given SelectStatement and returns an Executor.
+func (q *QueryExecutor) planStatement(stmt influxql.Statement, database string, chunkSize int) (Executor, error) {
+	switch stmt := stmt.(type) {
+	case *influxql.SelectStatement:
+		return q.PlanSelect(stmt, chunkSize)
+	case *influxql.ShowMeasurementsStatement:
+		return q.PlanShowMeasurements(stmt, database, chunkSize)
+	case *influxql.ShowTagKeysStatement:
+		return q.PlanShowTagKeys(stmt, database, chunkSize)
+	default:
+		return nil, fmt.Errorf("can't plan statement type: %v", stmt)
+	}
+}
+
+// PlanShowMeasurements creates an execution plan for a SHOW TAG KEYS statement and returns an Executor.
 func (q *QueryExecutor) PlanShowMeasurements(stmt *influxql.ShowMeasurementsStatement, database string, chunkSize int) (Executor, error) {
+	// Check for time in WHERE clause (not supported).
+	if influxql.HasTimeExpr(stmt.Condition) {
+		return nil, errors.New("SHOW MEASUREMENTS doesn't support time in WHERE clause")
+	}
+
 	// Get the database info.
-	di, err := q.MetaStore.Database(database)
+	di, err := q.MetaClient.Database(database)
 	if err != nil {
 		return nil, err
 	} else if di == nil {
@@ -598,87 +655,140 @@ func (q *QueryExecutor) PlanShowMeasurements(stmt *influxql.ShowMeasurementsStat
 	return executor, nil
 }
 
-func (q *QueryExecutor) executeShowMeasurementsStatement(statementID int, stmt *influxql.ShowMeasurementsStatement, database string, results chan *influxql.Result, chunkSize int) error {
+// PlanShowTagKeys creates an execution plan for a SHOW MEASUREMENTS statement and returns an Executor.
+func (q *QueryExecutor) PlanShowTagKeys(stmt *influxql.ShowTagKeysStatement, database string, chunkSize int) (Executor, error) {
+	// Check for time in WHERE clause (not supported).
+	if influxql.HasTimeExpr(stmt.Condition) {
+		return nil, errors.New("SHOW TAG KEYS doesn't support time in WHERE clause")
+	}
+
+	// Get the database info.
+	di, err := q.MetaClient.Database(database)
+	if err != nil {
+		return nil, err
+	} else if di == nil {
+		return nil, ErrDatabaseNotFound(database)
+	}
+
+	// Get info for all shards in the database.
+	shards := di.ShardInfos()
+
+	// Build the Mappers, one per shard.
+	mappers := []Mapper{}
+	for _, sh := range shards {
+		m, err := q.ShardMapper.CreateMapper(sh, stmt, chunkSize)
+		if err != nil {
+			return nil, err
+		}
+		if m == nil {
+			// No data for this shard, skip it.
+			continue
+		}
+		mappers = append(mappers, m)
+	}
+
+	executor := NewShowTagKeysExecutor(stmt, mappers, chunkSize)
+	return executor, nil
+}
+
+func (q *QueryExecutor) executeStatement(statementID int, stmt influxql.Statement, database string, results chan *influxql.Result, chunkSize int, closing chan struct{}) error {
 	// Plan statement execution.
-	e, err := q.PlanShowMeasurements(stmt, database, chunkSize)
+	e, err := q.planStatement(stmt, database, chunkSize)
 	if err != nil {
 		return err
 	}
 
 	// Execute plan.
-	ch := e.Execute()
-
+	ch := e.Execute(closing)
+	var writeerr error
+	var intoNum int64
+	var isinto bool
 	// Stream results from the channel. We should send an empty result if nothing comes through.
 	resultSent := false
 	for row := range ch {
+		// We had a write error. Continue draining results from the channel
+		// so we don't hang the goroutine in the executor.
+		if writeerr != nil {
+			continue
+		}
 		if row.Err != nil {
 			return row.Err
 		}
-		resultSent = true
-		results <- &influxql.Result{StatementID: statementID, Series: []*influxql.Row{row}}
+		selectstmt, ok := stmt.(*influxql.SelectStatement)
+		if ok && selectstmt.Target != nil {
+			isinto = true
+			// this is a into query. Write results back to database
+			writeerr = q.writeInto(row, selectstmt)
+			intoNum += int64(len(row.Values))
+		} else {
+			resultSent = true
+			results <- &influxql.Result{StatementID: statementID, Series: []*models.Row{row}}
+		}
+	}
+	if writeerr != nil {
+		return writeerr
+	} else if isinto {
+		results <- &influxql.Result{
+			StatementID: statementID,
+			Series: []*models.Row{{
+				Name: "result",
+				// it seems weird to give a time here, but so much stuff breaks if you don't
+				Columns: []string{"time", "written"},
+				Values: [][]interface{}{{
+					time.Unix(0, 0).UTC(),
+					intoNum,
+				}},
+			}},
+		}
+		return nil
 	}
 
 	if !resultSent {
-		results <- &influxql.Result{StatementID: statementID, Series: make([]*influxql.Row, 0)}
+		results <- &influxql.Result{StatementID: statementID, Series: make([]*models.Row, 0)}
 	}
 
 	return nil
 }
 
-func (q *QueryExecutor) executeShowTagKeysStatement(stmt *influxql.ShowTagKeysStatement, database string) *influxql.Result {
-	// Find the database.
-	db := q.Store.DatabaseIndex(database)
-	if db == nil {
-		return &influxql.Result{}
+func (q *QueryExecutor) writeInto(row *models.Row, selectstmt *influxql.SelectStatement) error {
+	// It might seem a bit weird that this is where we do this, since we will have to
+	// convert rows back to points. The Executors (both aggregate and raw) are complex
+	// enough that changing them to write back to the DB is going to be clumsy
+	//
+	// it might seem weird to have the write be in the QueryExecutor, but the interweaving of
+	// limitedRowWriter and ExecuteAggregate/Raw makes it ridiculously hard to make sure that the
+	// results will be the same as when queried normally.
+	measurement := intoMeasurement(selectstmt)
+	if measurement == "" {
+		measurement = row.Name
 	}
-
-	// Expand regex expressions in the FROM clause.
-	sources, err := q.expandSources(stmt.Sources)
+	intodb, err := intoDB(selectstmt)
 	if err != nil {
-		return &influxql.Result{Err: err}
+		return err
 	}
-
-	// Get the list of measurements we're interested in.
-	measurements, err := measurementsFromSourcesOrDB(db, sources...)
+	rp := intoRP(selectstmt)
+	points, err := convertRowToPoints(measurement, row)
 	if err != nil {
-		return &influxql.Result{Err: err}
+		return err
 	}
-
-	// Make result.
-	result := &influxql.Result{
-		Series: make(influxql.Rows, 0, len(measurements)),
+	req := &IntoWriteRequest{
+		Database:        intodb,
+		RetentionPolicy: rp,
+		Points:          points,
 	}
-
-	// Add one row per measurement to the result.
-	for _, m := range measurements {
-		// TODO: filter tag keys by stmt.Condition
-
-		// Get the tag keys in sorted order.
-		keys := m.TagKeys()
-
-		// Convert keys to an [][]interface{}.
-		values := make([][]interface{}, 0, len(m.seriesByTagKeyValue))
-		for _, k := range keys {
-			v := interface{}(k)
-			values = append(values, []interface{}{v})
-		}
-
-		// Make a result row for the measurement.
-		r := &influxql.Row{
-			Name:    m.Name,
-			Columns: []string{"tagKey"},
-			Values:  values,
-		}
-
-		result.Series = append(result.Series, r)
+	err = q.IntoWriter.WritePointsInto(req)
+	if err != nil {
+		return err
 	}
-
-	// TODO: LIMIT & OFFSET
-
-	return result
+	return nil
 }
 
 func (q *QueryExecutor) executeShowTagValuesStatement(stmt *influxql.ShowTagValuesStatement, database string) *influxql.Result {
+	// Check for time in WHERE clause (not supported).
+	if influxql.HasTimeExpr(stmt.Condition) {
+		return &influxql.Result{Err: errors.New("SHOW TAG VALUES doesn't support time in WHERE clause")}
+	}
+
 	// Find the database.
 	db := q.Store.DatabaseIndex(database)
 	if db == nil {
@@ -699,7 +809,7 @@ func (q *QueryExecutor) executeShowTagValuesStatement(stmt *influxql.ShowTagValu
 
 	// Make result.
 	result := &influxql.Result{
-		Series: make(influxql.Rows, 0),
+		Series: make(models.Rows, 0),
 	}
 
 	tagValues := make(map[string]stringSet)
@@ -734,7 +844,7 @@ func (q *QueryExecutor) executeShowTagValuesStatement(stmt *influxql.ShowTagValu
 	}
 
 	for k, v := range tagValues {
-		r := &influxql.Row{
+		r := &models.Row{
 			Name:    k + "TagValues",
 			Columns: []string{k},
 		}
@@ -776,13 +886,13 @@ func (q *QueryExecutor) executeShowFieldKeysStatement(stmt *influxql.ShowFieldKe
 
 	// Make result.
 	result := &influxql.Result{
-		Series: make(influxql.Rows, 0, len(measurements)),
+		Series: make(models.Rows, 0, len(measurements)),
 	}
 
 	// Loop through measurements, adding a result row for each.
 	for _, m := range measurements {
 		// Create a new row.
-		r := &influxql.Row{
+		r := &models.Row{
 			Name:    m.Name,
 			Columns: []string{"fieldKey"},
 		}
@@ -814,7 +924,7 @@ func measurementsFromSourcesOrDB(db *DatabaseIndex, sources ...influxql.Source)
 			if m, ok := source.(*influxql.Measurement); ok {
 				measurement := db.measurements[m.Name]
 				if measurement == nil {
-					return nil, ErrMeasurementNotFound(m.Name)
+					continue
 				}
 
 				measurements = append(measurements, measurement)
@@ -855,22 +965,6 @@ func (q *QueryExecutor) normalizeStatement(stmt influxql.Statement, defaultDatab
 			prefixes[n.Name] = n.Name
 		}
 	})
-	if err != nil {
-		return err
-	}
-
-	// Replace all variable references that used measurement prefixes.
-	influxql.WalkFunc(stmt, func(n influxql.Node) {
-		switch n := n.(type) {
-		case *influxql.VarRef:
-			for k, v := range prefixes {
-				if strings.HasPrefix(n.Val, k+".") {
-					n.Val = v + "." + influxql.QuoteIdent(n.Val[len(k)+1:])
-				}
-			}
-		}
-	})
-
 	return
 }
 
@@ -894,7 +988,7 @@ func (q *QueryExecutor) normalizeMeasurement(m *influxql.Measurement, defaultDat
 	}
 
 	// Find database.
-	di, err := q.MetaStore.Database(m.Database)
+	di, err := q.MetaClient.Database(m.Database)
 	if err != nil {
 		return err
 	} else if di == nil {
@@ -949,3 +1043,9 @@ var (
 func ErrDatabaseNotFound(name string) error { return fmt.Errorf("database not found: %s", name) }
 
 func ErrMeasurementNotFound(name string) error { return fmt.Errorf("measurement not found: %s", name) }
+
+type uint64Slice []uint64
+
+func (a uint64Slice) Len() int           { return len(a) }
+func (a uint64Slice) Swap(i, j int)      { a[i], a[j] = a[j], a[i] }
+func (a uint64Slice) Less(i, j int) bool { return a[i] < a[j] }
